from PyQt4 import Qt, QtGui, QtCore
from nupic_studio.ui import Global

class OutputForm(QtGui.QWidget):

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    QtGui.QWidget.__init__(self)

    self.initUI()

  #endregion

  #region Methods

  def initUI(self):

    # textBoxOutput
    self.textBoxOutput = QtGui.QTextEdit()
    self.textBoxOutput.setReadOnly(True)
    self.textBoxOutput.setAlignment(QtCore.Qt.AlignLeft)

    # layout
    layout = QtGui.QHBoxLayout()
    layout.addWidget(self.textBoxOutput)

    # OutputForm
    self.setLayout(layout)
    self.setWindowTitle("Output")
    self.setWindowIcon(QtGui.QIcon(Global.appPath + '/images/logo.ico'))
    self.setMinimumHeight(200)
    self.setMaximumHeight(300)

  def clearControls(self):
    """
    Reset all controls.
    """

    self.textBoxOutput.setText("")

  def addText(self, text):
    """
    Refresh controls for each time step.
    """

    self.textBoxOutput.append(text)
    cursor = self.textBoxOutput.textCursor()
    cursor.movePosition(QtGui.QTextCursor.End)
    self.textBoxOutput.setTextCursor(cursor)
    self.textBoxOutput.ensureCursorVisible()

  #endregion



import os
import sys
import time
from PyQt4 import Qt, QtCore, QtGui, QtOpenGL
from nupic_studio.project import Project
from nupic_studio.ui import Global
from nupic_studio.ui.start_form import StartForm
from nupic_studio.ui.main_form import MainForm
from nupic_studio.ui.architecture_form import ArchitectureForm
from nupic_studio.ui.node_information_form import NodeInformationForm
from nupic_studio.ui.simulation_form import SimulationForm
from nupic_studio.ui.output_form import OutputForm

def main():
  Global.app = QtGui.QApplication(sys.argv)
  Global.app.setStyleSheet("QGroupBox { border: 1px solid gray; } QGroupBox::title { padding: 0 5px; }")

  Global.appPath = os.path.abspath(os.path.join(__file__, '..'))
  Global.loadConfig()

  Global.project = Project()
  Global.simulationForm = SimulationForm()
  Global.architectureForm = ArchitectureForm()
  Global.nodeInformationForm = NodeInformationForm()
  Global.outputForm = OutputForm()
  Global.mainForm = MainForm()

  # Create and display the splash screen
  start = time.time()
  splash_pix = QtGui.QPixmap(Global.appPath + '/images/splash.png')
  splash = QtGui.QSplashScreen(splash_pix, QtCore.Qt.WindowStaysOnTopHint)
  splash.setMask(splash_pix.mask())
  splash.show()
  while time.time() - start < 3:
    time.sleep(0.001)
    Global.app.processEvents()
  splash.close()

  # Show start form
  startForm = StartForm()
  startForm.show()

  deploymentBuild = os.getenv("NUPIC_STUDIO_DEPLOYMENT_BUILD", False)
  if deploymentBuild:
    sys.exit(0)
  else:
    sys.exit(Global.app.exec_())


if __name__ == '__main__':
  main()



from PyQt4 import QtGui, QtCore

class NodeType:
  """
  Types of nodes in the hierarchy.
  """

  region = 1
  sensor = 2

class Node:
  """
  Node that represents region/sensors and their params.
  """

  #region Constructor

  def __init__(self, name, type):
    """
    Initializes a new instance of this class.
    """

    #region Instance fields

    self.name = name
    """The name of the Node."""

    self.type = type
    """Type of the node (Region or Sensor)"""

    self.width = 64
    """Width determines the number of columns in the X axis"""

    self.height = 64
    """Height determines the number of columns in the Y axis"""

    self._output = []
    """An array representing the current output from this node."""

    #region 2d-tree properties (node tree form)

    self.tree2d_x = 0.
    self.tree2d_y = 0.
    self.tree2d_polygon = QtGui.QPolygon()

    #endregion

    #region 3d-tree properties (simulation form)

    self.tree3d_x = 0
    self.tree3d_y = 0
    self.tree3d_z = 0

    #endregion

    #endregion

  #endregion

  #region Methods

  def initialize(self):
    """
    Initialize this node.
    """

    pass

  def nextStep(self):
    """
    Perfoms actions related to time step progression.
    """

    pass

  def getOutput(self):
    """
    Get output from this node.
    """

    return self._output

  #endregion



import os
import collections
from PyQt4 import QtGui, QtCore
from nupic_studio.htm import maxPreviousStepsWithInference

class View:
  """
  A class only to group properties related to pre-defined views.
  """

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    #region Instance fields

    self.menu = None
    self.name = ""
    self.showBitsNone = False
    self.showBitsActive = True
    self.showBitsPredicted = True
    self.showBitsFalselyPredicted = True
    self.showCellsNone = False
    self.showCellsLearning = True
    self.showCellsActive = True
    self.showCellsPredicted = True
    self.showCellsFalselyPredicted = True
    self.showCellsInactive = True
    self.showProximalSegmentsNone = False
    self.showProximalSegmentsActive = True
    self.showProximalSegmentsPredicted = True
    self.showProximalSegmentsFalselyPredicted = True
    self.showProximalSynapsesNone = False
    self.showProximalSynapsesConnected = True
    self.showProximalSynapsesActive = True
    self.showProximalSynapsesPredicted = True
    self.showProximalSynapsesFalselyPredicted = True
    self.showDistalSegmentsNone = False
    self.showDistalSegmentsActive = True
    self.showDistalSynapsesNone = False
    self.showDistalSynapsesConnected = True
    self.showDistalSynapsesActive = True

    #endregion

  #endregion

class Global:
  appPath = ''
  version = '0.1.0'

  simulationInitialized = False
  currStep = 0
  selStep = 0
  timeStepsPredictionsChart = None
  output = []

  views = []

  project = None
  architectureForm = None
  nodeInformationForm = None
  simulationForm = None
  outputForm = None
  mainForm = None

  @staticmethod
  def loadConfig():
    """
    Loads the content from XML file to config the program.
    """

    fileName = os.path.join(Global.appPath, "nupic_studio.config")
    file = QtCore.QFile(fileName)
    if (file.open(QtCore.QIODevice.ReadOnly)):
      xmlReader = QtCore.QXmlStreamReader()
      xmlReader.setDevice(file)
      while (not xmlReader.isEndDocument()):
        if xmlReader.isStartElement():
          if xmlReader.name().toString() == 'View':
            view = View()
            view.name = Global.__getStringAttribute(xmlReader.attributes(), 'name')
            view.showBitsNone = Global.__getBooleanAttribute(xmlReader.attributes(), 'showBitsNone')
            view.showBitsActive = Global.__getBooleanAttribute(xmlReader.attributes(), 'showBitsActive')
            view.showBitsPredicted = Global.__getBooleanAttribute(xmlReader.attributes(), 'showBitsPredicted')
            view.showBitsFalselyPredicted = Global.__getBooleanAttribute(xmlReader.attributes(), 'showBitsFalselyPredicted')
            view.showCellsNone = Global.__getBooleanAttribute(xmlReader.attributes(), 'showCellsNone')
            view.showCellsLearning = Global.__getBooleanAttribute(xmlReader.attributes(), 'showCellsLearning')
            view.showCellsActive = Global.__getBooleanAttribute(xmlReader.attributes(), 'showCellsActive')
            view.showCellsPredicted = Global.__getBooleanAttribute(xmlReader.attributes(), 'showCellsPredicted')
            view.showCellsFalselyPredicted = Global.__getBooleanAttribute(xmlReader.attributes(), 'showCellsFalselyPredicted')
            view.showCellsInactive = Global.__getBooleanAttribute(xmlReader.attributes(), 'showCellsInactive')
            view.showProximalSegmentsNone = Global.__getBooleanAttribute(xmlReader.attributes(), 'showProximalSegmentsNone')
            view.showProximalSegmentsActive = Global.__getBooleanAttribute(xmlReader.attributes(), 'showProximalSegmentsActive')
            view.showProximalSegmentsPredicted = Global.__getBooleanAttribute(xmlReader.attributes(), 'showProximalSegmentsPredicted')
            view.showProximalSegmentsFalselyPredicted = Global.__getBooleanAttribute(xmlReader.attributes(), 'showProximalSegmentsFalselyPredicted')
            view.showProximalSynapsesNone = Global.__getBooleanAttribute(xmlReader.attributes(), 'showProximalSynapsesNone')
            view.showProximalSynapsesConnected = Global.__getBooleanAttribute(xmlReader.attributes(), 'showProximalSynapsesConnected')
            view.showProximalSynapsesActive = Global.__getBooleanAttribute(xmlReader.attributes(), 'showProximalSynapsesActive')
            view.showProximalSynapsesPredicted = Global.__getBooleanAttribute(xmlReader.attributes(), 'showProximalSynapsesPredicted')
            view.showProximalSynapsesFalselyPredicted = Global.__getBooleanAttribute(xmlReader.attributes(), 'showProximalSynapsesFalselyPredicted')
            view.showDistalSegmentsNone = Global.__getBooleanAttribute(xmlReader.attributes(), 'showDistalSegmentsNone')
            view.showDistalSegmentsActive = Global.__getBooleanAttribute(xmlReader.attributes(), 'showDistalSegmentsActive')
            view.showDistalSynapsesNone = Global.__getBooleanAttribute(xmlReader.attributes(), 'showDistalSynapsesNone')
            view.showDistalSynapsesConnected = Global.__getBooleanAttribute(xmlReader.attributes(), 'showDistalSynapsesConnected')
            view.showDistalSynapsesActive = Global.__getBooleanAttribute(xmlReader.attributes(), 'showDistalSynapsesActive')
            Global.views.append(view)
        xmlReader.readNext()
      if (xmlReader.hasError()):
        QtGui.QMessageBox.critical(None, "Critical", "Ocurred a XML error: " + xmlReader.errorString().data(), QtGui.QMessageBox.Ok | QtGui.QMessageBox.Default, QtGui.QMessageBox.NoButton)
    else:
      QtGui.QMessageBox.critical(None, "Critical", "Cannot read the config file!", QtGui.QMessageBox.Ok | QtGui.QMessageBox.Default, QtGui.QMessageBox.NoButton)

  @staticmethod
  def __getStringAttribute(attributes, attributeName):
    if attributes.value(attributeName).toString() != "":
      attributeValue = str(attributes.value(attributeName).toString())
    else:
      attributeValue = ""
    return attributeValue

  @staticmethod
  def __getIntegerAttribute(attributes, attributeName):
    attributeValue = 0
    if attributes.value(attributeName).toString() != "":
      attributeValue = int(attributes.value(attributeName).toString())
    return attributeValue

  @staticmethod
  def __getFloatAttribute(attributes, attributeName):
    attributeValue = 0.0
    if attributes.value(attributeName).toString() != "":
      attributeValue = float(attributes.value(attributeName).toString())
    return attributeValue

  @staticmethod
  def __getBooleanAttribute(attributes, attributeName):
    attributeValue = False
    if attributes.value(attributeName).toString() == "True":
      attributeValue = True
    return attributeValue

  @staticmethod
  def saveConfig():
    """
    Saves the content from current program's configuration.
    """

    fileName = os.path.join(Global.appPath, "nupic_studio.config")
    file = QtCore.QFile(fileName)
    file.open(QtCore.QIODevice.WriteOnly)
    xmlWriter = QtCore.QXmlStreamWriter(file)
    xmlWriter.setAutoFormatting(True)
    xmlWriter.writeStartDocument()
    xmlWriter.writeStartElement('Program')

    for view in Global.views:
      xmlWriter.writeStartElement('View')
      xmlWriter.writeAttribute('name', view.name)
      xmlWriter.writeAttribute('showBitsNone',  str(view.showBitsNone))
      xmlWriter.writeAttribute('showBitsActive',  str(view.showBitsActive))
      xmlWriter.writeAttribute('showBitsPredicted',  str(view.showBitsPredicted))
      xmlWriter.writeAttribute('showBitsFalselyPredicted',  str(view.showBitsFalselyPredicted))
      xmlWriter.writeAttribute('showCellsNone',  str(view.showCellsNone))
      xmlWriter.writeAttribute('showCellsLearning',  str(view.showCellsLearning))
      xmlWriter.writeAttribute('showCellsActive',  str(view.showCellsActive))
      xmlWriter.writeAttribute('showCellsPredicted',  str(view.showCellsPredicted))
      xmlWriter.writeAttribute('showCellsFalselyPredicted',  str(view.showCellsFalselyPredicted))
      xmlWriter.writeAttribute('showCellsInactive',  str(view.showCellsInactive))
      xmlWriter.writeAttribute('showProximalSegmentsNone',  str(view.showProximalSegmentsNone))
      xmlWriter.writeAttribute('showProximalSegmentsActive',  str(view.showProximalSegmentsActive))
      xmlWriter.writeAttribute('showProximalSegmentsPredicted',  str(view.showProximalSegmentsPredicted))
      xmlWriter.writeAttribute('showProximalSegmentsFalselyPredicted',  str(view.showProximalSegmentsFalselyPredicted))
      xmlWriter.writeAttribute('showProximalSynapsesNone',  str(view.showProximalSynapsesNone))
      xmlWriter.writeAttribute('showProximalSynapsesConnected',  str(view.showProximalSynapsesConnected))
      xmlWriter.writeAttribute('showProximalSynapsesActive',  str(view.showProximalSynapsesActive))
      xmlWriter.writeAttribute('showProximalSynapsesPredicted',  str(view.showProximalSynapsesPredicted))
      xmlWriter.writeAttribute('showProximalSynapsesFalselyPredicted',  str(view.showProximalSynapsesFalselyPredicted))
      xmlWriter.writeAttribute('showDistalSegmentsNone',  str(view.showDistalSegmentsNone))
      xmlWriter.writeAttribute('showDistalSegmentsActive',  str(view.showDistalSegmentsActive))
      xmlWriter.writeAttribute('showDistalSynapsesNone',  str(view.showDistalSynapsesNone))
      xmlWriter.writeAttribute('showDistalSynapsesConnected',  str(view.showDistalSynapsesConnected))
      xmlWriter.writeAttribute('showDistalSynapsesActive',  str(view.showDistalSynapsesActive))
      xmlWriter.writeEndElement()

    xmlWriter.writeEndElement()
    xmlWriter.writeEndDocument()
    file.close()



import copy
from PyQt4 import QtGui, QtCore
from nupic_studio.htm.node_sensor import DataSourceType, PredictionsMethod
from nupic_studio.ui import Global
from nupic_studio.ui.node_sensor_encoding_form import EncodingForm

class SensorForm(QtGui.QDialog):

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    QtGui.QDialog.__init__(self)

    self.initUI()

    #region Instance fields

    self.encodings = []
    """Temporary list of encodings that is being edited"""

    self.encodingsChanged = False
    """Flag to indicate if encodings list was edited"""

    #endregion

  #endregion

  #region Methods

  def initUI(self):

    # labelSensorWidth
    self.labelSensorWidth = QtGui.QLabel()
    self.labelSensorWidth.setText("Sensor Width")
    self.labelSensorWidth.setAlignment(QtCore.Qt.AlignRight)

    # spinnerSensorWidth
    self.spinnerSensorWidth = QtGui.QSpinBox()
    self.spinnerSensorWidth.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerSensorWidth.setToolTip("Number of output bits in the X direction for this sensor.")
    self.spinnerSensorWidth.setMaximum(1000)
    self.spinnerSensorWidth.setEnabled(not Global.simulationInitialized)

    # labelSensorHeight
    self.labelSensorHeight = QtGui.QLabel()
    self.labelSensorHeight.setText("Sensor Height")
    self.labelSensorHeight.setAlignment(QtCore.Qt.AlignRight)

    # spinnerSensorHeight
    self.spinnerSensorHeight = QtGui.QSpinBox()
    self.spinnerSensorHeight.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerSensorHeight.setToolTip("Number of output bits in the Y direction for this sensor.")
    self.spinnerSensorHeight.setMaximum(1000)
    self.spinnerSensorHeight.setEnabled(not Global.simulationInitialized)

    # radioButtonDataSourceFile
    self.radioButtonDataSourceFile = QtGui.QRadioButton()
    self.radioButtonDataSourceFile.setText("File")
    self.radioButtonDataSourceFile.toggled.connect(self.__radioButtonDataSource_Click)

    # labelFile
    self.labelFile = QtGui.QLabel()
    self.labelFile.setText("File:")
    self.labelFile.setAlignment(QtCore.Qt.AlignRight)

    # textBoxFile
    self.textBoxFile = QtGui.QLineEdit()

    # buttonBrowseFile
    self.buttonBrowseFile = QtGui.QPushButton()
    self.buttonBrowseFile.setText("Browse...")
    self.buttonBrowseFile.clicked.connect(self.__buttonBrowseFile_Click)

    # radioButtonDataSourceDatabase
    self.radioButtonDataSourceDatabase = QtGui.QRadioButton()
    self.radioButtonDataSourceDatabase.setText("Database")
    self.radioButtonDataSourceDatabase.toggled.connect(self.__radioButtonDataSource_Click)

    # labelDatabaseConnectionString
    self.labelDatabaseConnectionString = QtGui.QLabel()
    self.labelDatabaseConnectionString.setText("Connection String:")
    self.labelDatabaseConnectionString.setAlignment(QtCore.Qt.AlignRight)

    # textBoxDatabaseConnectionString
    self.textBoxDatabaseConnectionString = QtGui.QLineEdit()

    # labelDatabaseTable
    self.labelDatabaseTable = QtGui.QLabel()
    self.labelDatabaseTable.setText("Table:")
    self.labelDatabaseTable.setAlignment(QtCore.Qt.AlignRight)

    # textBoxDatabaseTable
    self.textBoxDatabaseTable = QtGui.QLineEdit()

    # groupBoxDataSourceTypeLayout
    groupBoxDataSourceTypeLayout = QtGui.QGridLayout()
    groupBoxDataSourceTypeLayout.addWidget(self.radioButtonDataSourceFile, 0, 0)
    groupBoxDataSourceTypeLayout.addWidget(self.labelFile, 1, 0)
    groupBoxDataSourceTypeLayout.addWidget(self.textBoxFile, 1, 1)
    groupBoxDataSourceTypeLayout.addWidget(self.buttonBrowseFile, 2, 1)
    groupBoxDataSourceTypeLayout.addWidget(self.radioButtonDataSourceDatabase, 3, 0)
    groupBoxDataSourceTypeLayout.addWidget(self.labelDatabaseConnectionString, 4, 0)
    groupBoxDataSourceTypeLayout.addWidget(self.textBoxDatabaseConnectionString, 4, 1)
    groupBoxDataSourceTypeLayout.addWidget(self.labelDatabaseTable, 5, 0)
    groupBoxDataSourceTypeLayout.addWidget(self.textBoxDatabaseTable, 5, 1)

    # groupBoxDataSourceType
    self.groupBoxDataSourceType = QtGui.QGroupBox()
    self.groupBoxDataSourceType.setLayout(groupBoxDataSourceTypeLayout)
    self.groupBoxDataSourceType.setTitle("Data Source Type")
    self.groupBoxDataSourceType.setEnabled(not Global.simulationInitialized)

    # labelPredictionsMethod
    self.labelPredictionsMethod = QtGui.QLabel()
    self.labelPredictionsMethod.setText("Predictions Method:")
    self.labelPredictionsMethod.setAlignment(QtCore.Qt.AlignRight)

    # comboBoxPredictionsMethod
    self.comboBoxPredictionsMethod = QtGui.QComboBox()
    self.comboBoxPredictionsMethod.addItem(PredictionsMethod.reconstruction)
    self.comboBoxPredictionsMethod.addItem(PredictionsMethod.classification)
    self.comboBoxPredictionsMethod.setEnabled(not Global.simulationInitialized)

    # buttonNewEncoding
    self.buttonNewEncoding = QtGui.QPushButton()
    self.buttonNewEncoding.setText("New...")
    self.buttonNewEncoding.clicked.connect(self.__buttonNewEncoding_Click)
    self.buttonNewEncoding.setEnabled(not Global.simulationInitialized)

    # buttonEditEncoding
    self.buttonEditEncoding = QtGui.QPushButton()
    self.buttonEditEncoding.setText("Edit...")
    self.buttonEditEncoding.clicked.connect(self.__buttonEditEncoding_Click)
    self.buttonEditEncoding.setEnabled(not Global.simulationInitialized)

    # buttonDeleteEncoding
    self.buttonDeleteEncoding = QtGui.QPushButton()
    self.buttonDeleteEncoding.setText("Delete")
    self.buttonDeleteEncoding.clicked.connect(self.__buttonDeleteEncoding_Click)
    self.buttonDeleteEncoding.setEnabled(not Global.simulationInitialized)

    # listBoxEncodings
    self.listBoxEncodings = QtGui.QListWidget()
    self.listBoxEncodings.setEnabled(not Global.simulationInitialized)

    # encodingsButtonsLayout
    encodingsButtonsLayout = QtGui.QHBoxLayout()
    encodingsButtonsLayout.addWidget(self.buttonNewEncoding)
    encodingsButtonsLayout.addWidget(self.buttonEditEncoding)
    encodingsButtonsLayout.addWidget(self.buttonDeleteEncoding)

    # groupBoxEncodingsLayout
    groupBoxEncodingsLayout = QtGui.QVBoxLayout()
    groupBoxEncodingsLayout.addLayout(encodingsButtonsLayout)
    groupBoxEncodingsLayout.addWidget(self.listBoxEncodings)

    # groupBoxEncodings
    self.groupBoxEncodings = QtGui.QGroupBox()
    self.groupBoxEncodings.setLayout(groupBoxEncodingsLayout)
    self.groupBoxEncodings.setTitle("Encodings")

    # groupBoxMainLayout
    groupBoxMainLayout = QtGui.QGridLayout()
    groupBoxMainLayout.addWidget(self.labelSensorWidth, 0, 0)
    groupBoxMainLayout.addWidget(self.spinnerSensorWidth, 0, 1)
    groupBoxMainLayout.addWidget(self.labelSensorHeight, 1, 0)
    groupBoxMainLayout.addWidget(self.spinnerSensorHeight, 1, 1)
    groupBoxMainLayout.addWidget(self.groupBoxDataSourceType, 2, 1)
    groupBoxMainLayout.addWidget(self.labelPredictionsMethod, 3, 0)
    groupBoxMainLayout.addWidget(self.comboBoxPredictionsMethod, 3, 1)
    groupBoxMainLayout.addWidget(self.groupBoxEncodings, 4, 1)

    # groupBoxMain
    self.groupBoxMain = QtGui.QGroupBox()
    self.groupBoxMain.setLayout(groupBoxMainLayout)

    # buttonBox
    self.buttonBox = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Ok | QtGui.QDialogButtonBox.Cancel)
    self.buttonBox.button(QtGui.QDialogButtonBox.Ok).clicked.connect(self.__buttonOk_Click)
    self.buttonBox.button(QtGui.QDialogButtonBox.Ok).setEnabled(not Global.simulationInitialized)
    self.buttonBox.button(QtGui.QDialogButtonBox.Cancel).clicked.connect(self.__buttonCancel_Click)

    # layout
    layout = QtGui.QVBoxLayout()
    layout.addWidget(self.groupBoxMain)
    layout.addWidget(self.buttonBox)

    # SensorForm
    self.setLayout(layout)
    self.setModal(True)
    self.setWindowTitle("Sensor Properties")
    self.setWindowIcon(QtGui.QIcon(Global.appPath + '/images/logo.ico'))
    self.resize(400, 200)

  def setControlsValues(self):
    """
    Set controls values from a class instance.
    """

    # Set controls value with sensor params
    node = Global.architectureForm.designPanel.underMouseNode
    self.spinnerSensorWidth.setValue(node.width)
    self.spinnerSensorHeight.setValue(node.height)

    if node.dataSourceType == DataSourceType.file:
      self.radioButtonDataSourceFile.setChecked(True)
      self.textBoxFile.setText(node.fileName)
    elif node.dataSourceType == DataSourceType.database:
      self.radioButtonDataSourceDatabase.setChecked(True)
      self.textBoxDatabaseConnectionString.setText(node.databaseConnectionString)
      self.textBoxDatabaseTable.setText(node.databaseTable)

    self.comboBoxPredictionsMethod.setCurrentIndex(self.comboBoxPredictionsMethod.findText(node.predictionsMethod, QtCore.Qt.MatchFixedString))

    self.encodings = copy.deepcopy(node.encodings)
    self.__updateEncodingsListBox()

  def __updateEncodingsListBox(self):

    # Update the list box with the updated encodings
    self.listBoxEncodings.clear()
    for encoding in self.encodings:
      name = encoding.encoderFieldName.split('.')[0]
      self.listBoxEncodings.addItem(name)

    # Update controls according to list state
    if self.listBoxEncodings.count() > 0:
      if self.listBoxEncodings.currentRow() == -1:
        self.listBoxEncodings.setCurrentRow(0)
      self.buttonEditEncoding.setEnabled(not Global.simulationInitialized)
      self.buttonDeleteEncoding.setEnabled(not Global.simulationInitialized)
    else:
      self.buttonEditEncoding.setEnabled(False)
      self.buttonDeleteEncoding.setEnabled(False)

  #endregion

  #region Events

  def __buttonOk_Click(self, event):
    """
    Check if values changed and save the,.
    """

    if self.radioButtonDataSourceFile.isChecked() and self.textBoxFile.text() == '':
      QtGui.QMessageBox.warning(self, "Warning", "Input stream file was not specified.")
      return
    elif self.radioButtonDataSourceDatabase.isChecked() and self.textBoxDatabaseConnectionString.text() == '':
      QtGui.QMessageBox.warning(self, "Warning", "Database connection string was not specified.")
      return
    elif self.radioButtonDataSourceDatabase.isChecked() and self.textBoxDatabaseTable.text() == '':
      QtGui.QMessageBox.warning(self, "Warning", "Database table was not specified.")
      return
    elif self.listBoxEncodings.count() == 0:
      QtGui.QMessageBox.warning(self, "Warning", "Encodings list is empty. At least one encoding should be specified.")
      return
    else:
      hasFieldsWithInference = False
      for encoding in self.encodings:
        if encoding.enableInference:
          hasFieldsWithInference = True
          break
      if not hasFieldsWithInference:
        QtGui.QMessageBox.warning(self, "Warning", "At least one encoding should have inference enabled.")
        return

    width = self.spinnerSensorWidth.value()
    height = self.spinnerSensorHeight.value()
    dataSourceType = None
    if self.radioButtonDataSourceFile.isChecked():
      dataSourceType = DataSourceType.file
    elif self.radioButtonDataSourceDatabase.isChecked():
      dataSourceType = DataSourceType.database
    fileName = str(self.textBoxFile.text())
    databaseConnectionString = str(self.textBoxDatabaseConnectionString.text())
    databaseTable = str(self.textBoxDatabaseTable.text())
    predictionsMethod = str(self.comboBoxPredictionsMethod.currentText())

    # If anything has changed
    node = Global.architectureForm.designPanel.underMouseNode
    if node.width != width or node.height != height or node.predictionsMethod != predictionsMethod or node.dataSourceType != dataSourceType or node.fileName != fileName or node.databaseConnectionString != databaseConnectionString or node.databaseTable != databaseTable or self.encodingsChanged:
      # Set sensor params with controls values
      node.width = width
      node.height = height
      node.predictionsMethod = predictionsMethod
      node.dataSourceType = dataSourceType
      node.fileName = fileName
      node.databaseConnectionString = databaseConnectionString
      node.databaseTable = databaseTable
      node.encodings = copy.deepcopy(self.encodings)
      self.accept()

    self.close()

  def __buttonCancel_Click(self, event):
    self.reject()
    self.close()

  def __buttonBrowseFile_Click(self, event):
    # Ask user for an existing file
    selectedFile = QtGui.QFileDialog().getOpenFileName(self, "Open File", Global.appPath + '/projects', "Input files (*.csv)")

    # If file exists, set data source file
    if selectedFile != '':
      # Set file
      self.textBoxFile.setText(selectedFile)

  def __radioButtonDataSource_Click(self, event):
    if not Global.simulationInitialized:
      flag = self.radioButtonDataSourceFile.isChecked()
      self.textBoxFile.setEnabled(flag)
      self.buttonBrowseFile.setEnabled(flag)
      self.textBoxDatabaseConnectionString.setEnabled(not flag)
      self.textBoxDatabaseTable.setEnabled(not flag)

  def __buttonNewEncoding_Click(self, event):
    encodingForm = EncodingForm()
    encodingForm.encodingIdx = -1
    encodingForm.encodings = self.encodings
    dialogResult = encodingForm.exec_()
    if dialogResult == QtGui.QDialog.Accepted:
      self.encodingsChanged = True
      self.__updateEncodingsListBox()

  def __buttonEditEncoding_Click(self, event):
    encodingForm = EncodingForm()
    encodingForm.encodingIdx = self.listBoxEncodings.currentRow()
    encodingForm.encodings = self.encodings
    encodingForm.setControlsValues()
    dialogResult = encodingForm.exec_()
    if dialogResult == QtGui.QDialog.Accepted:
      self.encodingsChanged = True
      self.__updateEncodingsListBox()

  def __buttonDeleteEncoding_Click(self, event):
    self.encodings.remove(self.encodings[self.listBoxEncodings.currentRow()])
    self.encodingsChanged = True
    self.__updateEncodingsListBox()

  #endregion



from PyQt4 import QtGui, QtCore
from nupic_studio.ui import Global

class RegionForm(QtGui.QDialog):

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    QtGui.QDialog.__init__(self)

    self.initUI()

  #endregion

  #region Methods

  def initUI(self):

    # labelWidth
    self.labelWidth = QtGui.QLabel()
    self.labelWidth.setText("Width")
    self.labelWidth.setAlignment(QtCore.Qt.AlignRight)

    # spinnerWidth
    self.spinnerWidth = QtGui.QSpinBox()
    self.spinnerWidth.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerWidth.setMinimum(3)
    self.spinnerWidth.setEnabled(not Global.simulationInitialized)
    self.spinnerWidth.setToolTip("")

    # labelHeight
    self.labelHeight = QtGui.QLabel()
    self.labelHeight.setText("Height")
    self.labelHeight.setAlignment(QtCore.Qt.AlignRight)

    # spinnerHeight
    self.spinnerHeight = QtGui.QSpinBox()
    self.spinnerHeight.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerHeight.setMinimum(3)
    self.spinnerHeight.setEnabled(not Global.simulationInitialized)
    self.spinnerHeight.setToolTip("")

    # mainLayout
    mainLayout = QtGui.QGridLayout()
    mainLayout.addWidget(self.labelWidth, 0, 0)
    mainLayout.addWidget(self.spinnerWidth, 0, 1)
    mainLayout.addWidget(self.labelHeight, 1, 0)
    mainLayout.addWidget(self.spinnerHeight, 1, 1)

    # labelPotentialRadius
    self.labelPotentialRadius = QtGui.QLabel()
    self.labelPotentialRadius.setText("Potential Radius")
    self.labelPotentialRadius.setAlignment(QtCore.Qt.AlignRight)

    # spinnerPotentialRadius
    self.spinnerPotentialRadius = QtGui.QSpinBox()
    self.spinnerPotentialRadius.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerPotentialRadius.setEnabled(not Global.simulationInitialized)
    self.spinnerPotentialRadius.setToolTip("This parameter determines the extent of the input that each column can potentially be connected to. This can be thought of as the input bits that are visible to each column, or a 'receptiveField' of the field of vision. A large enough value will result in 'global coverage', meaning that each column can potentially be connected to every input bit. This parameter defines a square (or hyper square) area: a column will have a max square potential pool with sides of length 2 * potentialRadius + 1.")

    # labelPotentialPct
    self.labelPotentialPct = QtGui.QLabel()
    self.labelPotentialPct.setText("Potential (%)")
    self.labelPotentialPct.setAlignment(QtCore.Qt.AlignRight)

    # spinnerPotentialPct
    self.spinnerPotentialPct = QtGui.QDoubleSpinBox()
    self.spinnerPotentialPct.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerPotentialPct.setMaximum(1)
    self.spinnerPotentialPct.setDecimals(2)
    self.spinnerPotentialPct.setSingleStep(0.01)
    self.spinnerPotentialPct.setEnabled(not Global.simulationInitialized)
    self.spinnerPotentialPct.setToolTip("The percent of the inputs, within a column's potential radius, that a column can be connected to. If set to 1, the column will be connected to every input within its potential radius. This parameter is used to give each column a unique potential pool when a large potentialRadius causes overlap between the columns. At initialization time we choose ((2*potentialRadius + 1)^(# inputDimensions) * potentialPct) input bits to comprise the column's potential pool.")

    # checkBoxGlobalInhibition
    self.checkBoxGlobalInhibition = QtGui.QCheckBox()
    self.checkBoxGlobalInhibition.setText("Global Inhibition")
    self.checkBoxGlobalInhibition.setEnabled(not Global.simulationInitialized)
    self.checkBoxGlobalInhibition.setToolTip("If true, then during inhibition phase the winning columns are selected as the most active columns from the region as a whole. Otherwise, the winning columns are selected with respect to their local neighborhoods. Using global inhibition boosts performance x60.")

    # labelLocalAreaDensity
    self.labelLocalAreaDensity = QtGui.QLabel()
    self.labelLocalAreaDensity.setText("Local Area Density")
    self.labelLocalAreaDensity.setAlignment(QtCore.Qt.AlignRight)

    # spinnerLocalAreaDensity
    self.spinnerLocalAreaDensity = QtGui.QDoubleSpinBox()
    self.spinnerLocalAreaDensity.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerLocalAreaDensity.setMinimum(-1.0)
    self.spinnerLocalAreaDensity.setDecimals(2)
    self.spinnerLocalAreaDensity.setSingleStep(0.01)
    self.spinnerLocalAreaDensity.setEnabled(not Global.simulationInitialized)
    self.spinnerLocalAreaDensity.setToolTip("The desired density of active columns within a local inhibition area (the size of which is set by the internally calculated inhibitionRadius, which is in turn determined from the average size of the connected potential pools of all columns). The inhibition logic will insure that at most N columns remain ON within a local inhibition area, where N = localAreaDensity * (total number of columns in inhibition area).")

    # labelNumActiveColumnsPerInhArea
    self.labelNumActiveColumnsPerInhArea = QtGui.QLabel()
    self.labelNumActiveColumnsPerInhArea.setText("Num. Active Columns Per Inh. Area")
    self.labelNumActiveColumnsPerInhArea.setAlignment(QtCore.Qt.AlignRight)

    # spinnerNumActiveColumnsPerInhArea
    self.spinnerNumActiveColumnsPerInhArea = QtGui.QSpinBox()
    self.spinnerNumActiveColumnsPerInhArea.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerNumActiveColumnsPerInhArea.setEnabled(not Global.simulationInitialized)
    self.spinnerNumActiveColumnsPerInhArea.setToolTip("An alternate way to control the density of the active columns. If numActiveColumnsPerInhArea is specified then localAreaDensity must be less than 0, and vice versa. When using numActiveColumnsPerInhArea, the inhibition logic will insure that at most 'numActiveColumnsPerInhArea' columns remain ON within a local inhibition area (the size of which is set by the internally calculated inhibitionRadius, which is in turn determined from the average size of the connected receptive fields of all columns). When using this method, as columns learn and grow their effective receptive fields, the inhibitionRadius will grow, and hence the net density of the active columns will *decrease*. This is in contrast to the localAreaDensity method, which keeps the density of active columns the same regardless of the size of their receptive fields.")

    # labelStimulusThreshold
    self.labelStimulusThreshold = QtGui.QLabel()
    self.labelStimulusThreshold.setText("Stimulus Threshold")
    self.labelStimulusThreshold.setAlignment(QtCore.Qt.AlignRight)

    # spinnerStimulusThreshold
    self.spinnerStimulusThreshold = QtGui.QSpinBox()
    self.spinnerStimulusThreshold.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerStimulusThreshold.setEnabled(not Global.simulationInitialized)
    self.spinnerStimulusThreshold.setToolTip("This is a number specifying the minimum number of synapses that must be on in order for a columns to turn ON. The purpose of this is to prevent noise input from activating columns. Specified as a percent of a fully grown synapse.")

    # labelProximalSynConnectedPerm
    self.labelProximalSynConnectedPerm = QtGui.QLabel()
    self.labelProximalSynConnectedPerm.setText("Connected")
    self.labelProximalSynConnectedPerm.setAlignment(QtCore.Qt.AlignRight)

    # spinnerProximalSynConnectedPerm
    self.spinnerProximalSynConnectedPerm = QtGui.QDoubleSpinBox()
    self.spinnerProximalSynConnectedPerm.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerProximalSynConnectedPerm.setDecimals(4)
    self.spinnerProximalSynConnectedPerm.setSingleStep(0.0001)
    self.spinnerProximalSynConnectedPerm.setEnabled(not Global.simulationInitialized)
    self.spinnerProximalSynConnectedPerm.setToolTip("The default connected threshold. Any synapse whose permanence value is above the connected threshold is a 'connected synapse', meaning it can contribute to the cell's firing.")

    # labelProximalSynPermIncrement
    self.labelProximalSynPermIncrement = QtGui.QLabel()
    self.labelProximalSynPermIncrement.setText("Increment")
    self.labelProximalSynPermIncrement.setAlignment(QtCore.Qt.AlignRight)

    # spinnerProximalSynPermIncrement
    self.spinnerProximalSynPermIncrement = QtGui.QDoubleSpinBox()
    self.spinnerProximalSynPermIncrement.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerProximalSynPermIncrement.setDecimals(4)
    self.spinnerProximalSynPermIncrement.setSingleStep(0.0001)
    self.spinnerProximalSynPermIncrement.setEnabled(not Global.simulationInitialized)
    self.spinnerProximalSynPermIncrement.setToolTip("The amount by which an active synapse is incremented in each round. Specified as a percent of a fully grown synapse.")

    # labelProximalSynPermDecrement
    self.labelProximalSynPermDecrement = QtGui.QLabel()
    self.labelProximalSynPermDecrement.setText("Decrement")
    self.labelProximalSynPermDecrement.setAlignment(QtCore.Qt.AlignRight)

    # spinnerProximalSynPermDecrement
    self.spinnerProximalSynPermDecrement = QtGui.QDoubleSpinBox()
    self.spinnerProximalSynPermDecrement.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerProximalSynPermDecrement.setDecimals(4)
    self.spinnerProximalSynPermDecrement.setSingleStep(0.0001)
    self.spinnerProximalSynPermDecrement.setEnabled(not Global.simulationInitialized)
    self.spinnerProximalSynPermDecrement.setToolTip("The amount by which an inactive synapse is decremented in each round. Specified as a percent of a fully grown synapse.")

    # groupBoxProximalSynPerm
    groupBoxProximalSynPermLayout = QtGui.QGridLayout()
    groupBoxProximalSynPermLayout.addWidget(self.labelProximalSynConnectedPerm, 0, 0)
    groupBoxProximalSynPermLayout.addWidget(self.spinnerProximalSynConnectedPerm, 0, 1)
    groupBoxProximalSynPermLayout.addWidget(self.labelProximalSynPermIncrement, 1, 0)
    groupBoxProximalSynPermLayout.addWidget(self.spinnerProximalSynPermIncrement, 1, 1)
    groupBoxProximalSynPermLayout.addWidget(self.labelProximalSynPermDecrement, 2, 0)
    groupBoxProximalSynPermLayout.addWidget(self.spinnerProximalSynPermDecrement, 2, 1)

    # groupBoxProximalSynPerm
    self.groupBoxProximalSynPerm = QtGui.QGroupBox()
    self.groupBoxProximalSynPerm.setLayout(groupBoxProximalSynPermLayout)
    self.groupBoxProximalSynPerm.setTitle("Proximal Synapses Permanence")

    # labelMinPctOverlapDutyCycle
    self.labelMinPctOverlapDutyCycle = QtGui.QLabel()
    self.labelMinPctOverlapDutyCycle.setText("Min. Overlap Duty Cycle (%)")
    self.labelMinPctOverlapDutyCycle.setAlignment(QtCore.Qt.AlignRight)

    # spinnerMinPctOverlapDutyCycle
    self.spinnerMinPctOverlapDutyCycle = QtGui.QDoubleSpinBox()
    self.spinnerMinPctOverlapDutyCycle.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerMinPctOverlapDutyCycle.setMaximum(100)
    self.spinnerMinPctOverlapDutyCycle.setDecimals(3)
    self.spinnerMinPctOverlapDutyCycle.setSingleStep(0.001)
    self.spinnerMinPctOverlapDutyCycle.setEnabled(not Global.simulationInitialized)
    self.spinnerMinPctOverlapDutyCycle.setToolTip("A number between 0 and 1.0, used to set a floor on how often a column should have at least stimulusThreshold active inputs. Periodically, each column looks at the overlap duty cycle of all other columns within its inhibition radius and sets its own internal minimal acceptable duty cycle to:\
      minPctDutyCycleBeforeInh * max(other columns' duty cycles).\
    On each iteration, any column whose overlap duty cycle falls below this computed value will get all of its permanence values boosted up by synPermActiveInc. Raising all permanences in response to a sub-par duty cycle before inhibition allows a cell to search for new inputs when either its previously learned inputs are no longer ever active, or when the vast majority of them have been 'hijacked' by other columns.")

    # labelMinPctActiveDutyCycle
    self.labelMinPctActiveDutyCycle = QtGui.QLabel()
    self.labelMinPctActiveDutyCycle.setText("Min. Active Duty Cycle (%)")
    self.labelMinPctActiveDutyCycle.setAlignment(QtCore.Qt.AlignRight)

    # spinnerMinPctActiveDutyCycle
    self.spinnerMinPctActiveDutyCycle = QtGui.QDoubleSpinBox()
    self.spinnerMinPctActiveDutyCycle.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerMinPctActiveDutyCycle.setMaximum(100)
    self.spinnerMinPctActiveDutyCycle.setDecimals(3)
    self.spinnerMinPctActiveDutyCycle.setSingleStep(0.001)
    self.spinnerMinPctActiveDutyCycle.setEnabled(not Global.simulationInitialized)
    self.spinnerMinPctActiveDutyCycle.setToolTip("A number between 0 and 1.0, used to set a floor on how often a column should be activate. Periodically, each column looks at the activity duty cycle of all other columns within its inhibition radius and sets its own internal minimal acceptable duty cycle to:\
      minPctDutyCycleAfterInh * max(other columns' duty cycles).\
    On each iteration, any column whose duty cycle after inhibition falls below this computed value will get its internal boost factor increased.")

    # labelDutyCyclePeriod
    self.labelDutyCyclePeriod = QtGui.QLabel()
    self.labelDutyCyclePeriod.setText("Duty Cycle Period")
    self.labelDutyCyclePeriod.setAlignment(QtCore.Qt.AlignRight)

    # spinnerDutyCyclePeriod
    self.spinnerDutyCyclePeriod = QtGui.QSpinBox()
    self.spinnerDutyCyclePeriod.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerDutyCyclePeriod.setMaximum(1000)
    self.spinnerDutyCyclePeriod.setEnabled(not Global.simulationInitialized)
    self.spinnerDutyCyclePeriod.setToolTip("The period used to calculate duty cycles. Higher values make it take longer to respond to changes in boost or synPerConnectedCell. Shorter values make it more unstable and likely to oscillate.")

    # labelMaxBoost
    self.labelMaxBoost = QtGui.QLabel()
    self.labelMaxBoost.setText("Max Boost")
    self.labelMaxBoost.setAlignment(QtCore.Qt.AlignRight)

    # spinnerMaxBoost
    self.spinnerMaxBoost = QtGui.QDoubleSpinBox()
    self.spinnerMaxBoost.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerMaxBoost.setDecimals(2)
    self.spinnerMaxBoost.setSingleStep(0.01)
    self.spinnerMaxBoost.setEnabled(not Global.simulationInitialized)
    self.spinnerMaxBoost.setToolTip("The maximum overlap boost factor. Each column's overlap gets multiplied by a boost factor before it gets considered for inhibition. The actual boost factor for a column is number between 1.0 and maxBoost. A boost factor of 1.0 is used if the duty cycle is >= minOverlapDutyCycle, maxBoost is used if the duty cycle is 0, and any duty cycle in between is linearly extrapolated from these 2 endpoints.")

    # labelSpSeed
    self.labelSpSeed = QtGui.QLabel()
    self.labelSpSeed.setText("Seed")
    self.labelSpSeed.setAlignment(QtCore.Qt.AlignRight)

    # spinnerSpSeed
    self.spinnerSpSeed = QtGui.QSpinBox()
    self.spinnerSpSeed.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerSpSeed.setMinimum(-1)
    self.spinnerSpSeed.setMaximum(5000)
    self.spinnerSpSeed.setEnabled(not Global.simulationInitialized)
    self.spinnerSpSeed.setToolTip("Seed for random values.")

    # tabPageSpatialLayout
    tabPageSpatialLayout = QtGui.QGridLayout()
    tabPageSpatialLayout.addWidget(self.labelPotentialRadius, 0, 0)
    tabPageSpatialLayout.addWidget(self.spinnerPotentialRadius, 0, 1)
    tabPageSpatialLayout.addWidget(self.labelPotentialPct, 1, 0)
    tabPageSpatialLayout.addWidget(self.spinnerPotentialPct, 1, 1)
    tabPageSpatialLayout.addWidget(self.checkBoxGlobalInhibition, 2, 0)
    tabPageSpatialLayout.addWidget(self.labelLocalAreaDensity, 3, 0)
    tabPageSpatialLayout.addWidget(self.spinnerLocalAreaDensity, 3, 1)
    tabPageSpatialLayout.addWidget(self.labelNumActiveColumnsPerInhArea, 4, 0)
    tabPageSpatialLayout.addWidget(self.spinnerNumActiveColumnsPerInhArea, 4, 1)
    tabPageSpatialLayout.addWidget(self.labelStimulusThreshold, 5, 0)
    tabPageSpatialLayout.addWidget(self.spinnerStimulusThreshold, 5, 1)
    tabPageSpatialLayout.addWidget(self.groupBoxProximalSynPerm, 6, 1)
    tabPageSpatialLayout.addWidget(self.labelMinPctOverlapDutyCycle, 7, 0)
    tabPageSpatialLayout.addWidget(self.spinnerMinPctOverlapDutyCycle, 7, 1)
    tabPageSpatialLayout.addWidget(self.labelMinPctActiveDutyCycle, 8, 0)
    tabPageSpatialLayout.addWidget(self.spinnerMinPctActiveDutyCycle, 8, 1)
    tabPageSpatialLayout.addWidget(self.labelDutyCyclePeriod, 9, 0)
    tabPageSpatialLayout.addWidget(self.spinnerDutyCyclePeriod, 9, 1)
    tabPageSpatialLayout.addWidget(self.labelMaxBoost, 10, 0)
    tabPageSpatialLayout.addWidget(self.spinnerMaxBoost, 10, 1)
    tabPageSpatialLayout.addWidget(self.labelSpSeed, 11, 0)
    tabPageSpatialLayout.addWidget(self.spinnerSpSeed, 11, 1)

    # tabPageSpatial
    self.tabPageSpatial = QtGui.QWidget()
    self.tabPageSpatial.setLayout(tabPageSpatialLayout)

    # labelNumCellsPerColumn
    self.labelNumCellsPerColumn = QtGui.QLabel()
    self.labelNumCellsPerColumn.setText("Num. Cells Per Column")
    self.labelNumCellsPerColumn.setAlignment(QtCore.Qt.AlignRight)

    # spinnerNumCellsPerColumn
    self.spinnerNumCellsPerColumn = QtGui.QSpinBox()
    self.spinnerNumCellsPerColumn.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerNumCellsPerColumn.setMinimum(1)
    self.spinnerNumCellsPerColumn.setEnabled(not Global.simulationInitialized)
    self.spinnerNumCellsPerColumn.setToolTip("Number of cells per column. More cells, more contextual information")

    # labelDistalSynInitialPerm
    self.labelDistalSynInitialPerm = QtGui.QLabel()
    self.labelDistalSynInitialPerm.setText("Initial")
    self.labelDistalSynInitialPerm.setAlignment(QtCore.Qt.AlignRight)

    # spinnerDistalSynInitialPerm
    self.spinnerDistalSynInitialPerm = QtGui.QDoubleSpinBox()
    self.spinnerDistalSynInitialPerm.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerDistalSynInitialPerm.setDecimals(4)
    self.spinnerDistalSynInitialPerm.setSingleStep(0.0001)
    self.spinnerDistalSynInitialPerm.setEnabled(not Global.simulationInitialized)
    self.spinnerDistalSynInitialPerm.setToolTip("The initial permanence of an distal synapse.")

    # labelDistalSynConnectedPerm
    self.labelDistalSynConnectedPerm = QtGui.QLabel()
    self.labelDistalSynConnectedPerm.setText("Connected")
    self.labelDistalSynConnectedPerm.setAlignment(QtCore.Qt.AlignRight)

    # spinnerDistalSynConnectedPerm
    self.spinnerDistalSynConnectedPerm = QtGui.QDoubleSpinBox()
    self.spinnerDistalSynConnectedPerm.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerDistalSynConnectedPerm.setDecimals(4)
    self.spinnerDistalSynConnectedPerm.setSingleStep(0.0001)
    self.spinnerDistalSynConnectedPerm.setEnabled(not Global.simulationInitialized)
    self.spinnerDistalSynConnectedPerm.setToolTip("The default connected threshold. Any synapse whose permanence value is above the connected threshold is a 'connected synapse', meaning it can contribute to the cell's firing.")

    # labelDistalSynPermIncrement
    self.labelDistalSynPermIncrement = QtGui.QLabel()
    self.labelDistalSynPermIncrement.setText("Increment")
    self.labelDistalSynPermIncrement.setAlignment(QtCore.Qt.AlignRight)

    # spinnerDistalSynPermIncrement
    self.spinnerDistalSynPermIncrement = QtGui.QDoubleSpinBox()
    self.spinnerDistalSynPermIncrement.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerDistalSynPermIncrement.setDecimals(4)
    self.spinnerDistalSynPermIncrement.setSingleStep(0.0001)
    self.spinnerDistalSynPermIncrement.setEnabled(not Global.simulationInitialized)
    self.spinnerDistalSynPermIncrement.setToolTip("The amount by which an active synapse is incremented in each round. Specified as a percent of a fully grown synapse.")

    # labelDistalSynPermDecrement
    self.labelDistalSynPermDecrement = QtGui.QLabel()
    self.labelDistalSynPermDecrement.setText("Decrement")
    self.labelDistalSynPermDecrement.setAlignment(QtCore.Qt.AlignRight)

    # spinnerDistalSynPermDecrement
    self.spinnerDistalSynPermDecrement = QtGui.QDoubleSpinBox()
    self.spinnerDistalSynPermDecrement.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerDistalSynPermDecrement.setDecimals(4)
    self.spinnerDistalSynPermDecrement.setSingleStep(0.0001)
    self.spinnerDistalSynPermDecrement.setEnabled(not Global.simulationInitialized)
    self.spinnerDistalSynPermDecrement.setToolTip("The amount by which an inactive synapse is decremented in each round. Specified as a percent of a fully grown synapse.")

    # groupBoxDistalSynPermLayout
    groupBoxDistalSynPermLayout = QtGui.QGridLayout()
    groupBoxDistalSynPermLayout.addWidget(self.labelDistalSynInitialPerm, 0, 0)
    groupBoxDistalSynPermLayout.addWidget(self.spinnerDistalSynInitialPerm, 0, 1)
    groupBoxDistalSynPermLayout.addWidget(self.labelDistalSynConnectedPerm, 1, 0)
    groupBoxDistalSynPermLayout.addWidget(self.spinnerDistalSynConnectedPerm, 1, 1)
    groupBoxDistalSynPermLayout.addWidget(self.labelDistalSynPermIncrement, 2, 0)
    groupBoxDistalSynPermLayout.addWidget(self.spinnerDistalSynPermIncrement, 2, 1)
    groupBoxDistalSynPermLayout.addWidget(self.labelDistalSynPermDecrement, 3, 0)
    groupBoxDistalSynPermLayout.addWidget(self.spinnerDistalSynPermDecrement, 3, 1)

    # groupBoxDistalSynPerm
    self.groupBoxDistalSynPerm = QtGui.QGroupBox()
    self.groupBoxDistalSynPerm.setLayout(groupBoxDistalSynPermLayout)
    self.groupBoxDistalSynPerm.setTitle("Distal Synapses Permanence")

    # labelMinThreshold
    self.labelMinThreshold = QtGui.QLabel()
    self.labelMinThreshold.setText("Min. Threshold")
    self.labelMinThreshold.setAlignment(QtCore.Qt.AlignRight)
    self.labelMinThreshold.setToolTip("If the number of synapses active on a segment is at least this threshold, it is selected as the best matching cell in a bursing column")

    # spinnerMinThreshold
    self.spinnerMinThreshold = QtGui.QSpinBox()
    self.spinnerMinThreshold.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerMinThreshold.setEnabled(not Global.simulationInitialized)
    self.spinnerMinThreshold.setToolTip("")

    # labelActivationThreshold
    self.labelActivationThreshold = QtGui.QLabel()
    self.labelActivationThreshold.setText("Activation Threshold")
    self.labelActivationThreshold.setAlignment(QtCore.Qt.AlignRight)

    # spinnerActivationThreshold
    self.spinnerActivationThreshold = QtGui.QSpinBox()
    self.spinnerActivationThreshold.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerActivationThreshold.setEnabled(not Global.simulationInitialized)
    self.spinnerActivationThreshold.setToolTip("If the number of active connected synapses on a segment is at least this threshold, the segment is said to be active")

    # labelMaxNumNewSynapses
    self.labelMaxNumNewSynapses = QtGui.QLabel()
    self.labelMaxNumNewSynapses.setText("Max. Num. New Synapses")
    self.labelMaxNumNewSynapses.setAlignment(QtCore.Qt.AlignRight)

    # spinnerMaxNumNewSynapses
    self.spinnerMaxNumNewSynapses = QtGui.QSpinBox()
    self.spinnerMaxNumNewSynapses.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerMaxNumNewSynapses.setEnabled(not Global.simulationInitialized)
    self.spinnerMaxNumNewSynapses.setToolTip("The maximum number of synapses added to a segment during learning")

    # labelTpSeed
    self.labelTpSeed = QtGui.QLabel()
    self.labelTpSeed.setText("Seed")
    self.labelTpSeed.setAlignment(QtCore.Qt.AlignRight)

    # spinnerTpSeed
    self.spinnerTpSeed = QtGui.QSpinBox()
    self.spinnerTpSeed.setAlignment(QtCore.Qt.AlignRight)
    self.spinnerTpSeed.setMinimum(-1)
    self.spinnerTpSeed.setMaximum(5000)
    self.spinnerTpSeed.setEnabled(not Global.simulationInitialized)
    self.spinnerTpSeed.setToolTip("Seed for random values.")

    # tabPageTemporalLayout
    tabPageTemporalLayout = QtGui.QGridLayout()
    tabPageTemporalLayout.addWidget(self.labelNumCellsPerColumn, 0, 0)
    tabPageTemporalLayout.addWidget(self.spinnerNumCellsPerColumn, 0, 1)
    tabPageTemporalLayout.addWidget(self.groupBoxDistalSynPerm, 1, 1)
    tabPageTemporalLayout.addWidget(self.labelMinThreshold, 2, 0)
    tabPageTemporalLayout.addWidget(self.spinnerMinThreshold, 2, 1)
    tabPageTemporalLayout.addWidget(self.labelActivationThreshold, 3, 0)
    tabPageTemporalLayout.addWidget(self.spinnerActivationThreshold, 3, 1)
    tabPageTemporalLayout.addWidget(self.labelMaxNumNewSynapses, 4, 0)
    tabPageTemporalLayout.addWidget(self.spinnerMaxNumNewSynapses, 4, 1)
    tabPageTemporalLayout.addWidget(self.labelTpSeed, 5, 0)
    tabPageTemporalLayout.addWidget(self.spinnerTpSeed, 5, 1)
    tabPageTemporalLayout.setRowStretch(6, 100)

    # tabPageTemporal
    self.tabPageTemporal = QtGui.QWidget()
    self.tabPageTemporal.setLayout(tabPageTemporalLayout)

    # tabControlMain
    self.tabControlMain = QtGui.QTabWidget()
    self.tabControlMain.addTab(self.tabPageSpatial, "Spatial Parameters")
    self.tabControlMain.addTab(self.tabPageTemporal, "Temporal Parameters")
    self.tabControlMain.selectedIndex = 1

    # buttonBox
    self.buttonBox = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Ok | QtGui.QDialogButtonBox.Cancel)
    self.buttonBox.button(QtGui.QDialogButtonBox.Ok).clicked.connect(self.__buttonOk_Click)
    self.buttonBox.button(QtGui.QDialogButtonBox.Ok).setEnabled(not Global.simulationInitialized)
    self.buttonBox.button(QtGui.QDialogButtonBox.Cancel).clicked.connect(self.__buttonCancel_Click)

    # layout
    layout = QtGui.QVBoxLayout()
    layout.addLayout(mainLayout)
    layout.addWidget(self.tabControlMain)
    layout.addWidget(self.buttonBox)

    # RegionForm
    self.setLayout(layout)
    self.setModal(True)
    self.setWindowTitle("Region Properties")
    self.setWindowIcon(QtGui.QIcon(Global.appPath + '/images/logo.ico'))

  def setControlsValues(self):
    """
    Set controls values from a class instance.
    """

    # Set controls value with region params
    node = Global.architectureForm.designPanel.underMouseNode
    self.spinnerWidth.setValue(node.width)
    self.spinnerHeight.setValue(node.height)
    self.spinnerPotentialRadius.setValue(node.potentialRadius)
    self.spinnerPotentialPct.setValue(node.potentialPct)
    self.checkBoxGlobalInhibition.setChecked(node.globalInhibition)
    self.spinnerLocalAreaDensity.setValue(node.localAreaDensity)
    self.spinnerNumActiveColumnsPerInhArea.setValue(node.numActiveColumnsPerInhArea)
    self.spinnerStimulusThreshold.setValue(node.stimulusThreshold)
    self.spinnerProximalSynConnectedPerm.setValue(node.proximalSynConnectedPerm)
    self.spinnerProximalSynPermIncrement.setValue(node.proximalSynPermIncrement)
    self.spinnerProximalSynPermDecrement.setValue(node.proximalSynPermDecrement)
    self.spinnerMinPctOverlapDutyCycle.setValue(node.minPctOverlapDutyCycle)
    self.spinnerMinPctActiveDutyCycle.setValue(node.minPctActiveDutyCycle)
    self.spinnerDutyCyclePeriod.setValue(node.dutyCyclePeriod)
    self.spinnerMaxBoost.setValue(node.maxBoost)
    self.spinnerSpSeed.setValue(node.spSeed)
    self.spinnerNumCellsPerColumn.setValue(node.numCellsPerColumn)
    self.spinnerDistalSynInitialPerm.setValue(node.distalSynInitialPerm)
    self.spinnerDistalSynConnectedPerm.setValue(node.distalSynConnectedPerm)
    self.spinnerDistalSynPermIncrement.setValue(node.distalSynPermIncrement)
    self.spinnerDistalSynPermDecrement.setValue(node.distalSynPermDecrement)
    self.spinnerMinThreshold.setValue(node.minThreshold)
    self.spinnerActivationThreshold.setValue(node.activationThreshold)
    self.spinnerMaxNumNewSynapses.setValue(node.maxNumNewSynapses)
    self.spinnerTpSeed.setValue(node.tpSeed)

  #endregion

  #region Events

  def __buttonOk_Click(self, event):
    """
    Check if values changed and save the,.
    """

    width = self.spinnerWidth.value()
    height = self.spinnerHeight.value()
    potentialRadius = self.spinnerPotentialRadius.value()
    potentialPct = self.spinnerPotentialPct.value()
    globalInhibition = self.checkBoxGlobalInhibition.isChecked()
    localAreaDensity = self.spinnerLocalAreaDensity.value()
    numActiveColumnsPerInhArea = self.spinnerNumActiveColumnsPerInhArea.value()
    stimulusThreshold = self.spinnerStimulusThreshold.value()
    proximalSynConnectedPerm = self.spinnerProximalSynConnectedPerm.value()
    proximalSynPermIncrement = self.spinnerProximalSynPermIncrement.value()
    proximalSynPermDecrement = self.spinnerProximalSynPermDecrement.value()
    minPctOverlapDutyCycle = self.spinnerMinPctOverlapDutyCycle.value()
    minPctActiveDutyCycle = self.spinnerMinPctActiveDutyCycle.value()
    dutyCyclePeriod = self.spinnerDutyCyclePeriod.value()
    maxBoost = self.spinnerMaxBoost.value()
    spSeed = self.spinnerSpSeed.value()
    numCellsPerColumn = self.spinnerNumCellsPerColumn.value()
    distalSynInitialPerm = self.spinnerDistalSynInitialPerm.value()
    distalSynConnectedPerm = self.spinnerDistalSynConnectedPerm.value()
    distalSynPermIncrement = self.spinnerDistalSynPermIncrement.value()
    distalSynPermDecrement = self.spinnerDistalSynPermDecrement.value()
    minThreshold = self.spinnerMinThreshold.value()
    activationThreshold = self.spinnerActivationThreshold.value()
    maxNumNewSynapses = self.spinnerMaxNumNewSynapses.value()
    tpSeed = self.spinnerTpSeed.value()

    # If anything has changed
    node = Global.architectureForm.designPanel.underMouseNode
    if  node.width != width or node.height != height or node.potentialRadius != potentialRadius or node.potentialPct != potentialPct or node.globalInhibition != globalInhibition or node.localAreaDensity != localAreaDensity or node.numActiveColumnsPerInhArea != numActiveColumnsPerInhArea or node.stimulusThreshold != stimulusThreshold\
      or node.proximalSynConnectedPerm != proximalSynConnectedPerm or node.proximalSynPermIncrement != proximalSynPermIncrement or node.proximalSynPermDecrement != proximalSynPermDecrement or node.minPctOverlapDutyCycle != minPctOverlapDutyCycle or node.minPctActiveDutyCycle != minPctActiveDutyCycle or node.dutyCyclePeriod != dutyCyclePeriod or node.maxBoost != maxBoost or node.spSeed != spSeed\
       or node.numCellsPerColumn != numCellsPerColumn or node.distalSynInitialPerm != distalSynInitialPerm or node.distalSynConnectedPerm != distalSynConnectedPerm or node.distalSynPermIncrement != distalSynPermIncrement or node.distalSynPermDecrement != distalSynPermDecrement or node.minThreshold != minThreshold or node.activationThreshold != activationThreshold or node.maxNumNewSynapses != maxNumNewSynapses or node.tpSeed != tpSeed:

      # Set region params with controls values
      node.width = width
      node.height = height
      node.potentialRadius = potentialRadius
      node.potentialPct = potentialPct
      node.globalInhibition = globalInhibition
      node.localAreaDensity = localAreaDensity
      node.numActiveColumnsPerInhArea = numActiveColumnsPerInhArea
      node.stimulusThreshold = stimulusThreshold
      node.proximalSynConnectedPerm = proximalSynConnectedPerm
      node.proximalSynPermIncrement = proximalSynPermIncrement
      node.proximalSynPermDecrement = proximalSynPermDecrement
      node.minPctOverlapDutyCycle = minPctOverlapDutyCycle
      node.minPctActiveDutyCycle = minPctActiveDutyCycle
      node.dutyCyclePeriod = dutyCyclePeriod
      node.maxBoost = maxBoost
      node.spSeed = spSeed
      node.numCellsPerColumn = numCellsPerColumn
      node.distalSynInitialPerm = distalSynInitialPerm
      node.distalSynConnectedPerm = distalSynConnectedPerm
      node.distalSynPermIncrement = distalSynPermIncrement
      node.distalSynPermDecrement = distalSynPermDecrement
      node.minThreshold = minThreshold
      node.activationThreshold = activationThreshold
      node.maxNumNewSynapses = maxNumNewSynapses
      node.tpSeed = tpSeed

      self.accept()

    self.close()

  def __buttonCancel_Click(self, event):
    self.reject()
    self.close()

  #endregion



from nupic_studio import MachineState
from nupic_studio.htm import maxPreviousSteps, maxFutureSteps, maxPreviousStepsWithInference
from nupic.algorithms.CLAClassifier import CLAClassifier

class FieldDataType:
  """
  Types of data which a raw input is composed.
  """

  boolean = "Boolean"
  integer = "Integer"
  decimal = "Decimal"
  dateTime = "DateTime"
  string = "String"

class Encoding:
  """
  A class only to group properties related to encodings.
  """

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    #region Instance fields

    self.dataSourceFieldName = ''
    """Target field of the database table or file."""

    self.dataSourceFieldDataType = FieldDataType.string
    """Data type of the field returned by database table or file."""

    self.encoder = None
    """Optional encoder to convert raw data to htm input and vice-versa."""

    self.encoderModule = ""
    """Module name which encoder class is imported."""

    self.encoderClass = ""
    """Class name which encode or decode values."""

    self.encoderParams = ""
    """Parameters passed to the encoder class constructor."""

    self.encoderFieldName = ""
    """Field name returned by encoder when decode() function."""

    self.encoderFieldDataType = FieldDataType.string
    """Data type of the field returned by encoder."""

    self.enableInference = False
    """Enable inference for this field."""

    self.currentValue = MachineState(None, maxPreviousSteps)
    """Value read currently from database."""

    self.bestPredictedValue = MachineState(None, maxPreviousSteps)
    """Best value predicted by network. This is need to build predictions chart."""

    self.predictedValues = MachineState(None, maxPreviousSteps)
    """Values predicted by network."""

    #endregion

  #endregion

  #region Methods

  def initialize(self):
    """
    Initialize this node.
    """

    # Create Classifier instance with appropriate parameters
    self.minProbabilityThreshold = 0.0001
    self.steps = []
    for step in range(maxFutureSteps):
      self.steps.append(step+1)
    self.classifier = CLAClassifier(steps=self.steps)

    # Increase history according to inference flag
    if self.enableInference:
      maxLen = maxPreviousStepsWithInference
      self.bestPredictedValue = MachineState(0, maxLen)
    else:
      maxLen = maxPreviousSteps
    self.currentValue = MachineState(0, maxLen)

  #endregion



import collections
import numpy
import pyqtgraph.opengl as gl
import pyqtgraph as pg
from PyQt4 import QtGui, QtCore
from nupic_studio.htm.node import Node, NodeType
from nupic_studio.htm.segment import SegmentType
from nupic_studio.ui import Global, View
from nupic_studio.ui.simulation_legend_form import SimulationLegendForm

class SimulationForm(QtGui.QWidget):

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    QtGui.QWidget.__init__(self)

    #region Instance fields

    # Views
    self.defaultViewMenu = None
    self.selectedViewMenu = None

    # Tree
    self.topRegion = None
    self.treeWidth = 0
    self.treeHeight = 0

    # Space to skip horizontally between siblings and vertically between generations
    self.offsetHorizontalNodes = 20
    self.offsetVerticalNodes = 25

    # Horizontal space between two columns
    self.offsetColumns = 10

    # Vertical space between two cells
    self.offsetCells = 5

    # Colors
    colorGray = QtGui.QColor.fromRgb(190, 190, 190)
    colorBlue = QtGui.QColor.fromRgb(0, 0, 255)
    colorGreen = QtGui.QColor.fromRgb(50, 205, 50)
    colorLightGreen = QtGui.QColor.fromRgb(125, 255, 0)
    colorYellow = QtGui.QColor.fromRgb(255, 215, 80)
    colorRed = QtGui.QColor.fromRgb(255, 0, 0)

    # General color scheme
    self.colorInactive = colorGray
    self.colorSelected = colorBlue

    # Sensor bit color scheme
    self.colorBitActive = colorGreen
    self.colorBitPredicted = colorYellow
    self.colorBitFalselyPredicted = colorRed

    # Cell color scheme
    self.colorCellLearning = colorLightGreen
    self.colorCellActive = colorGreen
    self.colorCellPredicted = colorYellow
    self.colorCellFalselyPredicted = colorRed

    # Segment color scheme
    self.colorSegmentActive = colorGreen
    self.colorSegmentPredicted = colorYellow
    self.colorSegmentFalselyPredicted = colorRed

    # Synapse color scheme
    self.colorSynapseConnected = colorGreen
    self.colorSynapsePredicted = colorYellow
    self.colorSynapseFalselyPredicted = colorRed

    #endregion

    self.initUI()

    # Create views menus
    for view in Global.views:
      view.menu = QtGui.QAction(self)
      view.menu.setText(view.name)
      view.menu.setCheckable(True)
      view.menu.triggered.connect(self.menuView_Click)
      self.menuViews.addAction(view.menu)

      # If this is the first menu, set it as default
      if self.defaultViewMenu == None:
        self.defaultViewMenu = view.menu

    # Load default view
    self.selectView(self.defaultViewMenu)

  #endregion

  #region Methods

  def initUI(self):

    # simulationViewer
    self.simulationViewer = gl.GLViewWidget()
    self.simulationViewer.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
    self.simulationViewer.customContextMenuRequested.connect(self.showContextMenu)

    # menuViewsNew
    self.menuViewsNew = QtGui.QAction(self)
    self.menuViewsNew.setText("Create new view")
    self.menuViewsNew.triggered.connect(self.menuViewsNew_Click)

    # menuViewsSave
    self.menuViewsSave = QtGui.QAction(self)
    self.menuViewsSave.setText("Save selected view")
    self.menuViewsSave.triggered.connect(self.menuViewsSave_Click)

    # menuViewsDelete
    self.menuViewsDelete = QtGui.QAction(self)
    self.menuViewsDelete.setText("Delete selected view")
    self.menuViewsDelete.triggered.connect(self.menuViewsDelete_Click)

    # menuViews
    self.menuViews = QtGui.QMenu()
    self.menuViews.addAction(self.menuViewsNew)
    self.menuViews.addAction(self.menuViewsSave)
    self.menuViews.addAction(self.menuViewsDelete)
    self.menuViews.addSeparator()
    self.menuViews.setTitle("&View")

    # menuCameraTop
    self.menuCameraTop = QtGui.QAction(self)
    self.menuCameraTop.setText("&Top")
    self.menuCameraTop.setShortcut('Ctrl+PgUp')
    self.menuCameraTop.triggered.connect(self.__updateCamera)

    # menuCameraBottom
    self.menuCameraBottom = QtGui.QAction(self)
    self.menuCameraBottom.setText("&Bottom")
    self.menuCameraBottom.setShortcut('Ctrl+PgDown')
    self.menuCameraBottom.triggered.connect(self.__updateCamera)

    # menuCameraPerspective
    self.menuCameraPerspective = QtGui.QAction(self)
    self.menuCameraPerspective.setText("&Perspective")
    self.menuCameraPerspective.setShortcut('Ctrl+End')
    self.menuCameraPerspective.triggered.connect(self.__updateCamera)

    # menuCameraDefault
    self.menuCameraDefault = QtGui.QAction(self)
    self.menuCameraDefault.setText("&Default")
    self.menuCameraDefault.setShortcut('Ctrl+Home')
    self.menuCameraDefault.triggered.connect(self.__updateCamera)

    # menuCamera
    self.menuCamera = QtGui.QMenu()
    self.menuCamera.addAction(self.menuCameraTop)
    self.menuCamera.addAction(self.menuCameraBottom)
    self.menuCamera.addAction(self.menuCameraPerspective)
    self.menuCamera.addAction(self.menuCameraDefault)
    self.menuCamera.setTitle("&Camera")

    # menuShowBitsNone
    self.menuShowBitsNone = QtGui.QAction(self)
    self.menuShowBitsNone.setText("&None")
    self.menuShowBitsNone.setCheckable(True)
    self.menuShowBitsNone.triggered.connect(self.menuShowBits_Click)

    # menuShowBitsActive
    self.menuShowBitsActive = QtGui.QAction(self)
    self.menuShowBitsActive.setText("&Active")
    self.menuShowBitsActive.setCheckable(True)
    self.menuShowBitsActive.triggered.connect(self.menuShowBits_Click)

    # menuShowBitsPredicted
    self.menuShowBitsPredicted = QtGui.QAction(self)
    self.menuShowBitsPredicted.setText("&Predicted")
    self.menuShowBitsPredicted.setCheckable(True)
    self.menuShowBitsPredicted.triggered.connect(self.menuShowBits_Click)

    # menuShowBitsFalselyPredicted
    self.menuShowBitsFalselyPredicted = QtGui.QAction(self)
    self.menuShowBitsFalselyPredicted.setText("&Falsely Predicted")
    self.menuShowBitsFalselyPredicted.setCheckable(True)
    self.menuShowBitsFalselyPredicted.triggered.connect(self.menuShowBits_Click)

    # menuShowBits
    self.menuShowBits = QtGui.QMenu()
    self.menuShowBits.addAction(self.menuShowBitsNone)
    self.menuShowBits.addAction(self.menuShowBitsActive)
    self.menuShowBits.addAction(self.menuShowBitsPredicted)
    self.menuShowBits.addAction(self.menuShowBitsFalselyPredicted)
    self.menuShowBits.setTitle("&Sensor bits")

    # menuShowCellsNone
    self.menuShowCellsNone = QtGui.QAction(self)
    self.menuShowCellsNone.setText("&None")
    self.menuShowCellsNone.setCheckable(True)
    self.menuShowCellsNone.triggered.connect(self.menuShowCells_Click)

    # menuShowCellsLearning
    self.menuShowCellsLearning = QtGui.QAction(self)
    self.menuShowCellsLearning.setText("&Learning")
    self.menuShowCellsLearning.setCheckable(True)
    self.menuShowCellsLearning.triggered.connect(self.menuShowCells_Click)

    # menuShowCellsActive
    self.menuShowCellsActive = QtGui.QAction(self)
    self.menuShowCellsActive.setText("&Active")
    self.menuShowCellsActive.setCheckable(True)
    self.menuShowCellsActive.triggered.connect(self.menuShowCells_Click)

    # menuShowCellsInactive
    self.menuShowCellsInactive = QtGui.QAction(self)
    self.menuShowCellsInactive.setText("&Inactive")
    self.menuShowCellsInactive.setCheckable(True)
    self.menuShowCellsInactive.triggered.connect(self.menuShowCells_Click)

    # menuShowCellsPredicted
    self.menuShowCellsPredicted = QtGui.QAction(self)
    self.menuShowCellsPredicted.setText("&Predicted")
    self.menuShowCellsPredicted.setCheckable(True)
    self.menuShowCellsPredicted.triggered.connect(self.menuShowCells_Click)

    # menuShowCellsFalselyPredicted
    self.menuShowCellsFalselyPredicted = QtGui.QAction(self)
    self.menuShowCellsFalselyPredicted.setText("&Falsely Predicted")
    self.menuShowCellsFalselyPredicted.setCheckable(True)
    self.menuShowCellsFalselyPredicted.triggered.connect(self.menuShowCells_Click)

    # menuShowCells
    self.menuShowCells = QtGui.QMenu()
    self.menuShowCells.addAction(self.menuShowCellsNone)
    self.menuShowCells.addAction(self.menuShowCellsLearning)
    self.menuShowCells.addAction(self.menuShowCellsActive)
    self.menuShowCells.addAction(self.menuShowCellsPredicted)
    self.menuShowCells.addAction(self.menuShowCellsFalselyPredicted)
    self.menuShowCells.addAction(self.menuShowCellsInactive)
    self.menuShowCells.setTitle("C&ells")

    # menuShowProximalSegmentsNone
    self.menuShowProximalSegmentsNone = QtGui.QAction(self)
    self.menuShowProximalSegmentsNone.setText("&None")
    self.menuShowProximalSegmentsNone.setCheckable(True)
    self.menuShowProximalSegmentsNone.triggered.connect(self.menuShowProximalSegments_Click)

    # menuShowProximalSegmentsActive
    self.menuShowProximalSegmentsActive = QtGui.QAction(self)
    self.menuShowProximalSegmentsActive.setText("&Active")
    self.menuShowProximalSegmentsActive.setCheckable(True)
    self.menuShowProximalSegmentsActive.triggered.connect(self.menuShowProximalSegments_Click)

    # menuShowProximalSegmentsPredicted
    self.menuShowProximalSegmentsPredicted = QtGui.QAction(self)
    self.menuShowProximalSegmentsPredicted.setText("&Predicted")
    self.menuShowProximalSegmentsPredicted.setCheckable(True)
    self.menuShowProximalSegmentsPredicted.triggered.connect(self.menuShowProximalSegments_Click)

    # menuShowProximalSegmentsFalselyPredicted
    self.menuShowProximalSegmentsFalselyPredicted = QtGui.QAction(self)
    self.menuShowProximalSegmentsFalselyPredicted.setText("&Falsely Predicted")
    self.menuShowProximalSegmentsFalselyPredicted.setCheckable(True)
    self.menuShowProximalSegmentsFalselyPredicted.triggered.connect(self.menuShowProximalSegments_Click)

    # menuShowProximalSegments
    self.menuShowProximalSegments = QtGui.QMenu()
    self.menuShowProximalSegments.addAction(self.menuShowProximalSegmentsNone)
    self.menuShowProximalSegments.addAction(self.menuShowProximalSegmentsActive)
    self.menuShowProximalSegments.addAction(self.menuShowProximalSegmentsPredicted)
    self.menuShowProximalSegments.addAction(self.menuShowProximalSegmentsFalselyPredicted)
    self.menuShowProximalSegments.setTitle("&Segments")

    # menuShowProximalSynapsesNone
    self.menuShowProximalSynapsesNone = QtGui.QAction(self)
    self.menuShowProximalSynapsesNone.setText("&None")
    self.menuShowProximalSynapsesNone.setCheckable(True)
    self.menuShowProximalSynapsesNone.triggered.connect(self.menuShowProximalSynapses_Click)

    # menuShowProximalSynapsesConnected
    self.menuShowProximalSynapsesConnected = QtGui.QAction(self)
    self.menuShowProximalSynapsesConnected.setText("&Connected")
    self.menuShowProximalSynapsesConnected.setCheckable(True)
    self.menuShowProximalSynapsesConnected.triggered.connect(self.menuShowProximalSynapses_Click)

    # menuShowProximalSynapsesActive
    self.menuShowProximalSynapsesActive = QtGui.QAction(self)
    self.menuShowProximalSynapsesActive.setText("&Active")
    self.menuShowProximalSynapsesActive.setCheckable(True)
    self.menuShowProximalSynapsesActive.triggered.connect(self.menuShowProximalSynapses_Click)

    # menuShowProximalSynapsesPredicted
    self.menuShowProximalSynapsesPredicted = QtGui.QAction(self)
    self.menuShowProximalSynapsesPredicted.setText("&Predicted")
    self.menuShowProximalSynapsesPredicted.setCheckable(True)
    self.menuShowProximalSynapsesPredicted.triggered.connect(self.menuShowProximalSynapses_Click)

    # menuShowProximalSynapsesFalselyPredicted
    self.menuShowProximalSynapsesFalselyPredicted = QtGui.QAction(self)
    self.menuShowProximalSynapsesFalselyPredicted.setText("&Falsely Predicted")
    self.menuShowProximalSynapsesFalselyPredicted.setCheckable(True)
    self.menuShowProximalSynapsesFalselyPredicted.triggered.connect(self.menuShowProximalSynapses_Click)

    # menuShowProximalSynapses
    self.menuShowProximalSynapses = QtGui.QMenu()
    self.menuShowProximalSynapses.addAction(self.menuShowProximalSynapsesNone)
    self.menuShowProximalSynapses.addAction(self.menuShowProximalSynapsesConnected)
    self.menuShowProximalSynapses.addAction(self.menuShowProximalSynapsesActive)
    self.menuShowProximalSynapses.addAction(self.menuShowProximalSynapsesPredicted)
    self.menuShowProximalSynapses.addAction(self.menuShowProximalSynapsesFalselyPredicted)
    self.menuShowProximalSynapses.setTitle("&Synapses")

    # menuShowProximal
    self.menuShowProximal = QtGui.QMenu()
    self.menuShowProximal.addMenu(self.menuShowProximalSegments)
    self.menuShowProximal.addMenu(self.menuShowProximalSynapses)
    self.menuShowProximal.setTitle("&Proximal")

    # menuShowDistalSegmentsNone
    self.menuShowDistalSegmentsNone = QtGui.QAction(self)
    self.menuShowDistalSegmentsNone.setText("&None")
    self.menuShowDistalSegmentsNone.setCheckable(True)
    self.menuShowDistalSegmentsNone.triggered.connect(self.menuShowDistalSegments_Click)

    # menuShowDistalSegmentsActive
    self.menuShowDistalSegmentsActive = QtGui.QAction(self)
    self.menuShowDistalSegmentsActive.setText("&Active")
    self.menuShowDistalSegmentsActive.setCheckable(True)
    self.menuShowDistalSegmentsActive.triggered.connect(self.menuShowDistalSegments_Click)

    # menuShowDistalSegments
    self.menuShowDistalSegments = QtGui.QMenu()
    self.menuShowDistalSegments.addAction(self.menuShowDistalSegmentsNone)
    self.menuShowDistalSegments.addAction(self.menuShowDistalSegmentsActive)
    self.menuShowDistalSegments.setTitle("&Segments")

    # menuShowDistalSynapsesNone
    self.menuShowDistalSynapsesNone = QtGui.QAction(self)
    self.menuShowDistalSynapsesNone.setText("&None")
    self.menuShowDistalSynapsesNone.setCheckable(True)
    self.menuShowDistalSynapsesNone.triggered.connect(self.menuShowDistalSynapses_Click)

    # menuShowDistalSynapsesConnected
    self.menuShowDistalSynapsesConnected = QtGui.QAction(self)
    self.menuShowDistalSynapsesConnected.setText("&Connected")
    self.menuShowDistalSynapsesConnected.setCheckable(True)
    self.menuShowDistalSynapsesConnected.triggered.connect(self.menuShowDistalSynapses_Click)

    # menuShowDistalSynapsesActive
    self.menuShowDistalSynapsesActive = QtGui.QAction(self)
    self.menuShowDistalSynapsesActive.setText("&Active")
    self.menuShowDistalSynapsesActive.setCheckable(True)
    self.menuShowDistalSynapsesActive.triggered.connect(self.menuShowDistalSynapses_Click)

    # menuShowDistalSynapses
    self.menuShowDistalSynapses = QtGui.QMenu()
    self.menuShowDistalSynapses.addAction(self.menuShowDistalSynapsesNone)
    self.menuShowDistalSynapses.addAction(self.menuShowDistalSynapsesConnected)
    self.menuShowDistalSynapses.addAction(self.menuShowDistalSynapsesActive)
    self.menuShowDistalSynapses.setTitle("&Synapses")

    # menuShowDistal
    self.menuShowDistal = QtGui.QMenu()
    self.menuShowDistal.addMenu(self.menuShowDistalSegments)
    self.menuShowDistal.addMenu(self.menuShowDistalSynapses)
    self.menuShowDistal.setTitle("&Distal")

    # menuShow
    self.menuShow = QtGui.QMenu()
    self.menuShow.addMenu(self.menuShowBits)
    self.menuShow.addMenu(self.menuShowCells)
    self.menuShow.addMenu(self.menuShowProximal)
    self.menuShow.addMenu(self.menuShowDistal)
    self.menuShow.setTitle("&Show")

    # menuLegend
    self.menuLegend = QtGui.QAction(self)
    self.menuLegend.setText("&Legend")
    self.menuLegend.triggered.connect(self.menuLegend_Click)

    # menuSimulation
    self.menuSimulation = QtGui.QMenu()
    self.menuSimulation.addMenu(self.menuViews)
    self.menuSimulation.addMenu(self.menuCamera)
    self.menuSimulation.addMenu(self.menuShow)
    self.menuSimulation.addAction(self.menuLegend)
    self.menuSimulation.setTitle("&Simulation")

    # layout
    layout = QtGui.QGridLayout()
    layout.addWidget(self.simulationViewer, 1, 0)
    layout.setRowStretch(1, 100)

    # SimulationForm
    self.setLayout(layout)
    self.setWindowTitle("Simulation")
    self.setWindowIcon(QtGui.QIcon(Global.appPath + '/images/logo.ico'))
    self.setMinimumWidth(400)
    self.setMinimumHeight(200)
    self.setToolTip("Left button drag: Rotates the scene around a central point.\r\nControl key + Middle button drag: Pan the scene by moving the central 'look-at' point within the plane.\r\nWheel spin: Zoom in/out.\r\nRight button: Shows menu.")

  def showContextMenu(self, pos):
    """
    Event handling right-click contextMenu
    """

    if Global.simulationInitialized:
      self.menuSimulation.exec_(self.mapToGlobal(pos))
    else:
      QtGui.QMessageBox.information(self, "Information", "Context menu available only during the simulation.")

  def __updateCamera(self, event):
    """
    Update camera position in the scene
    """

    menuClicked = self.sender()
    self.simulationViewer.setCameraPosition(distance = (self.treeHeight * 30))
    if menuClicked == self.menuCameraTop:
      self.simulationViewer.setCameraPosition(elevation = 90)
      self.simulationViewer.setCameraPosition(azimuth = 90)
    elif menuClicked == self.menuCameraBottom:
      self.simulationViewer.setCameraPosition(elevation = -90)
      self.simulationViewer.setCameraPosition(azimuth = 90)
    elif menuClicked == self.menuCameraPerspective:
      self.simulationViewer.setCameraPosition(elevation = 17)
      self.simulationViewer.setCameraPosition(azimuth = 45)
    else:
      self.simulationViewer.setCameraPosition(elevation = 17)
      self.simulationViewer.setCameraPosition(azimuth = 90)

  def clearControls(self):
    """
    Reset all controls.
    """

    # Remove all items
    while len(self.simulationViewer.items) > 0:
      self.simulationViewer.removeItem(self.simulationViewer.items[0])

    # Draw a sphere only to initialize simulation.
    # If we don't do this, viewer crashes. Probably a PYQtGraph bug
    fooMd = gl.MeshData.sphere(rows=10, cols=10)
    self.fooItem = gl.GLMeshItem(meshdata=fooMd, smooth=False, shader='shaded', glOptions='opaque')
    self.fooItem.translate(0, 0, 0)
    self.simulationViewer.addItem(self.fooItem)
    self.simulationViewer.setCameraPosition(distance = 10000)

  def initializeControls(self):
    """
    Refresh controls for each time step.
    """

    # Remove initial sphere
    self.simulationViewer.removeItem(self.fooItem)

    # Arrange the tree once to see how big it is.
    self.treeWidth = 0
    self.treeHeight = 0
    minX = 0
    minZ = 0
    minX, minZ = self.__arrangeNode(self.topRegion, minX, minZ)

    # Rearrange the tree again to center it horizontally.
    offsetX = self.topRegion.tree3d_x
    self.__centerNode(self.topRegion, offsetX)

    # Rearrange the tree again to invert it vertically.
    offsetZ = 50
    self.__invertNode(self.topRegion, offsetZ)

    # Once we have the final position of the regions, we can calculate the position of every column and cell.
    self.__calculateNodeElementsPosition(self.topRegion)

    # Draw the tree recursively from top region.
    self.__drawNode(self.topRegion, True)

    # Adjust camera
    self.__updateCamera(None)

  def refreshControls(self):
    """
    Refresh controls for each time step.
    """

    if Global.simulationInitialized:
      # Draw the tree recursively from top region.
      self.__drawNode(self.topRegion, False)

  def __centerNode(self, node, offsetX):
    """
    Rearrange the node in order to tree is in the center (0, 0, 0) of the scene.
    """

    node.tree3d_x -= offsetX
    for feeder in Global.project.network.getFeederNodes(node):
      self.__centerNode(feeder, offsetX)

  def __invertNode(self, node, offsetZ):
    """
    Rearrange the node in order to tree is inverted from top to bottom.
    """

    node.tree3d_z = offsetZ - node.tree3d_z
    for feeder in Global.project.network.getFeederNodes(node):
      self.__invertNode(feeder, offsetZ)

  def __calculateNodeElementsPosition(self, node):
    """
    Calculate the position of columns and cells of this node.
    """

    # Calculate the relative position of the first column (0, 0)
    # The purpose is we have symmetrical positions. If a node has 30 columns in X axis, then the relative X of the first column is -15 while the last is 15.
    x0 = (node.width / 2) * (-1)
    y0 = (node.height / 2) * (-1)

    # Calculate the absolute position of each column
    for y in range(node.height):
      for x in range(node.width):
        # The absolute X is calculated by multiply its relative position with offset
        xCol = node.tree3d_x + ((x0 + x) * self.offsetColumns)

        # The absolute Y is calculated by multiply its relative position with offset
        yCol = node.tree3d_y + ((y0 + y) * self.offsetColumns)

        # Calculate positions of the columns of cells if node is a region
        # or input bits if node is a sensor
        zCol = node.tree3d_z

        if node.type == NodeType.region:
          column = node.getColumn(x, y)
          column.tree3d_x = xCol
          column.tree3d_y = yCol
          column.tree3d_z = zCol

          # The proximal segment transverse all cells on this column
          column.segment.tree3d_x1 = xCol
          column.segment.tree3d_y1 = yCol
          column.segment.tree3d_z1 = zCol + ((node.numCellsPerColumn - 1) * self.offsetCells)
          column.segment.tree3d_x2 = xCol
          column.segment.tree3d_y2 = yCol
          column.segment.tree3d_z2 = zCol - self.offsetCells # Segment down towards to feeder nodes

          # Calculate the absolute position of each cell
          for z in range(len(column.cells)):
            # The absolute Z is calculated by multiply its relative position with offset
            cell = column.getCell(z)
            cell.tree3d_x = xCol
            cell.tree3d_y = yCol
            cell.tree3d_z = zCol + (z * self.offsetCells)
        else:
          bit = node.getBit(x, y)
          bit.tree3d_x = xCol
          bit.tree3d_y = yCol
          bit.tree3d_z = zCol

    # Perform the same actions for the lower nodes that feed this node
    for feeder in Global.project.network.getFeederNodes(node):
      self.__calculateNodeElementsPosition(feeder)

  def __arrangeNode(self, node, minX, minZ):
    """
    Arrange the node and the lower nodes that feed it in the allowed area.
    Set minX to indicate the right edge of our subtree.
    Set minZ to indicate the bottom edge of our subtree.
    """

    # See how big this node is.
    width = node.width * self.offsetColumns
    height = node.height
    if node.type == NodeType.region:
      depth = node.numCellsPerColumn * self.offsetCells
    else:
      depth = self.offsetCells

    # Recursively arrange the lower nodes that feed this node,
    # allowing room for this node.
    x = minX
    biggestMinZ = minZ + depth
    subtreeMinZ = minZ + depth + self.offsetVerticalNodes
    numFeeders = 0
    for feeder in Global.project.network.getFeederNodes(node):
      # Arrange this feeder's subtree.
      feederMinZ = subtreeMinZ
      x, feederMinZ = self.__arrangeNode(feeder, x, feederMinZ)

      # See if this increases the biggest minZ value.
      if biggestMinZ < feederMinZ:
        biggestMinZ = feederMinZ

      # Allow room before the next sibling.
      x += self.offsetHorizontalNodes

      numFeeders += 1

    # Remove the spacing after the last feeder.
    if numFeeders > 0:
      x -= self.offsetHorizontalNodes

    # See if this node is wider than the subtree under it.
    subtreeWidth = x - minX
    if width > subtreeWidth:
      # Center the subtree under this node.
      # Make the lower nodes that feed this node rearrange themselves
      # moved to center their subtrees.
      x = minX + (width - subtreeWidth) / 2
      for feeder in Global.project.network.getFeederNodes(node):
        # Arrange this feeder's subtree.
        x, subtreeMinZ = self.__arrangeNode(feeder, x, subtreeMinZ)

        # Allow room before the next sibling.
        x += self.offsetHorizontalNodes

      # The subtree's width is this node's width.
      subtreeWidth = width

    # Set this node's center position.
    node.tree3d_x = minX + (subtreeWidth / 2)
    node.tree3d_y = 0
    node.tree3d_z = minZ + (depth / 2)

    # Increase minX to allow room for the subtree before returning.
    minX += subtreeWidth

    if subtreeWidth > self.treeWidth:
      self.treeWidth = subtreeWidth

    if height > self.treeHeight:
      self.treeHeight = height

    # Set the return value for minZ.
    minZ = biggestMinZ

    return minX, minZ

  def __drawNode(self, node, initialize):
    """
    Draw the nodes for the subtree rooted at this node.
    """

    # Recursively make the node draw its feeders.
    for feeder in Global.project.network.getFeederNodes(node):
      self.__drawNode(feeder, initialize)

    # Draw a column of cells if node is a region
    # or an input bit if node is a sensor
    if node.type == NodeType.region:
      for column in node.columns:
        if initialize:
          for cell in column.cells:
            cell.tree3d_initialized = False
        self.__drawColumn(column)
    else:
      for bit in node.bits:
        if initialize:
          bit.tree3d_initialized = False
        self.__drawBit(bit)

  def __drawBit(self, bit):

    # Update properties according to state
    isVisible = True
    if self.menuShowBitsNone.isChecked():
      isVisible = False
    elif bit.isFalselyPredicted.atGivenStepAgo(Global.selStep) and self.menuShowBitsFalselyPredicted.isChecked():
      color = self.colorBitFalselyPredicted
    elif bit.isPredicted.atGivenStepAgo(Global.selStep) and self.menuShowBitsPredicted.isChecked():
      color = self.colorBitPredicted
    elif bit.isActive.atGivenStepAgo(Global.selStep) and self.menuShowBitsActive.isChecked():
      color = self.colorBitActive
    else:
      color = self.colorInactive

    if isVisible:
      # Draw the input bit
      if not bit.tree3d_initialized:
        # TODO: Use cube instead of sphere
        bitMd = gl.MeshData.sphere(rows=2, cols=4)
        bit.tree3d_item = gl.GLMeshItem(meshdata=bitMd, shader='shaded', smooth=False, glOptions='opaque')
        bit.tree3d_item.translate(bit.tree3d_x, bit.tree3d_y, bit.tree3d_z)
        bit.tree3d_initialized = True
        self.simulationViewer.addItem(bit.tree3d_item)

      # Update the color
      if bit.tree3d_selected:
        color = self.colorSelected
      bit.tree3d_item.setColor(color)

    if bit.tree3d_item != None:
      bit.tree3d_item.setVisible(isVisible)

  def __drawColumn(self, column):

    # Update proximal segment
    self.__drawSegment(column.segment)
    for cell in column.cells:
      self.__drawCell(cell)

  def __drawCell(self, cell):

    # Update properties according to state
    isVisible = True
    if self.menuShowCellsNone.isChecked():
      isVisible = False
    elif cell.isFalselyPredicted.atGivenStepAgo(Global.selStep) and self.menuShowCellsFalselyPredicted.isChecked():
      color = self.colorCellFalselyPredicted
    elif cell.isPredicted.atGivenStepAgo(Global.selStep) and self.menuShowCellsPredicted.isChecked():
      color = self.colorCellPredicted
    elif cell.isLearning.atGivenStepAgo(Global.selStep) and self.menuShowCellsLearning.isChecked():
      color = self.colorCellLearning
    elif cell.isActive.atGivenStepAgo(Global.selStep) and self.menuShowCellsActive.isChecked():
      color = self.colorCellActive
    elif self.menuShowCellsInactive.isChecked():
      color = self.colorInactive
    else:
      isVisible = False

    if isVisible:
      # Draw the cell
      if not cell.tree3d_initialized:
        cellMd = gl.MeshData.sphere(rows=10, cols=10)
        cell.tree3d_item = gl.GLMeshItem(meshdata=cellMd, shader='shaded', smooth=False, glOptions='opaque')
        cell.tree3d_item.translate(cell.tree3d_x, cell.tree3d_y, cell.tree3d_z)
        cell.tree3d_initialized = True
        self.simulationViewer.addItem(cell.tree3d_item)

      # Update the color
      if cell.tree3d_selected:
        color = self.colorSelected
      cell.tree3d_item.setColor(color)

    if cell.tree3d_item != None:
      cell.tree3d_item.setVisible(isVisible)

    # Draw/update all distal segments
    for segment in cell.segments:
      segment.tree3d_x1 = cell.tree3d_x
      segment.tree3d_y1 = cell.tree3d_y
      segment.tree3d_z1 = cell.tree3d_z
      segment.tree3d_x2, segment.tree3d_y2, segment.tree3d_z2 = self.__calculateSegmentEndPos(segment, segment.tree3d_x1, segment.tree3d_y1, segment.tree3d_z1)
      self.__drawSegment(segment)

  def __calculateSegmentEndPos(self, segment, xSeg1, ySeg1, zSeg1):
    """
    Calculates an average position of the segment's end through their synapses' end positions.
    """

    sumK = 0.
    numXBelow = 0
    numXAbove = 0
    for synapse in segment.synapses:
      xSyn = synapse.inputElem.tree3d_x
      ySyn = synapse.inputElem.tree3d_y

      # Calculate 'k' (slope) of the straight line representing this synapse
      deltaY = ySyn - ySeg1
      deltaX = xSyn - xSeg1
      if deltaX != 0:
        k = float(deltaY / deltaX)
      else:
        k = 3
      sumK += k

      if xSyn >= xSeg1:
        numXAbove += 1
      else:
        numXBelow += 1

    # Calculate direction of the straight line with base on the number of synapses X's below or above the segment X's
    if numXAbove >= numXBelow:
      direction = 1
    else:
      direction = -1

    # Calculate the 'k' (slope) of the new straight line representing this segment
    # It is an average value among the 'k' of the synapses
    k = 0
    if len(segment.synapses) > 0:
      k = int(sumK / len(segment.synapses))

    # Find the 'b' of the straight line equation using 'k' and segment's start position:
    #   y = ax + b (where 'a' = 'k')
    b = ySeg1 - (k * xSeg1)

    # Calculate the segment's end position
    # TODO: Optimize this routine, i.e. discard loop to find max value
    maxX = xSeg1 + ((self.offsetColumns / 3) * direction)
    maxY = ySeg1 + ((self.offsetColumns / 3) * direction)
    incX = (self.offsetColumns / 10) * direction
    xSeg2 = xSeg1
    while True:
      ySeg2 = (xSeg2 * k) + b
      if ((xSeg2 <= maxX or ySeg2 <= maxY) and direction < 0) or ((xSeg2 >= maxX or ySeg2 >= maxY) and direction > 0):
        break
      xSeg2 += incX
    zSeg2 = zSeg1

    return int(xSeg2), int(ySeg2), zSeg2

  def __drawSegment(self, segment):

    # Update properties according to state
    isVisible = True
    if segment.isRemoved.atGivenStepAgo(Global.selStep) or (segment.type == SegmentType.proximal and self.menuShowProximalSegmentsNone.isChecked()) or (segment.type == SegmentType.distal and self.menuShowDistalSegmentsNone.isChecked()):
      isVisible = False
    else:
      if segment.isFalselyPredicted.atGivenStepAgo(Global.selStep):
        if segment.type == SegmentType.proximal and self.menuShowProximalSegmentsFalselyPredicted.isChecked():
          color = self.colorSegmentFalselyPredicted
        else:
          isVisible = False
      elif segment.isPredicted.atGivenStepAgo(Global.selStep):
        if segment.type == SegmentType.proximal and self.menuShowProximalSegmentsPredicted.isChecked():
          color = self.colorSegmentPredicted
        else:
          isVisible = False
      elif segment.isActive.atGivenStepAgo(Global.selStep):
        if (segment.type == SegmentType.proximal and self.menuShowProximalSegmentsActive.isChecked()) or (segment.type == SegmentType.distal and self.menuShowDistalSegmentsActive.isChecked()):
          color = self.colorSegmentActive
        else:
          isVisible = False
      else:
        if segment.type == SegmentType.proximal:
          color = self.colorInactive
        else:
          isVisible = False

    if isVisible:
      # Draw the segment
      if not segment.tree3d_initialized:
        pts = numpy.array([[segment.tree3d_x1, segment.tree3d_y1, segment.tree3d_z1], [segment.tree3d_x2, segment.tree3d_y2, segment.tree3d_z2]])
        segment.tree3d_item = gl.GLLinePlotItem(pos=pts, width=1, antialias=False)
        segment.tree3d_initialized = True
        self.simulationViewer.addItem(segment.tree3d_item)

      # Update the color
      if segment.tree3d_selected:
        color = self.colorSelected
      segment.tree3d_item.color = pg.glColor(color)
    else:
      segment.tree3d_initialized = False
      if segment.tree3d_item in self.simulationViewer.items:
        self.simulationViewer.removeItem(segment.tree3d_item)

    # Draw/update all synapses of this segment
    for synapse in segment.synapses:
      self.__drawSynapse(segment, synapse, isVisible)

  def __drawSynapse(self, segment, synapse, segmentIsVisible):

    # Update properties according to state
    isVisible = True
    if synapse.isRemoved.atGivenStepAgo(Global.selStep) or (not segment.isActive.atGivenStepAgo(Global.selStep) and not segment.isPredicted.atGivenStepAgo(Global.selStep) and not segment.isFalselyPredicted.atGivenStepAgo(Global.selStep)) or (segment.type == SegmentType.proximal and self.menuShowProximalSynapsesNone.isChecked()) or (segment.type == SegmentType.distal and self.menuShowDistalSynapsesNone.isChecked()):
      isVisible = False
    else:
      if synapse.isFalselyPredicted.atGivenStepAgo(Global.selStep):
        if (segment.type == SegmentType.proximal and self.menuShowProximalSynapsesFalselyPredicted.isChecked()):
          color = self.colorSynapseFalselyPredicted
        else:
          isVisible = False
      elif synapse.isPredicted.atGivenStepAgo(Global.selStep):
        if (segment.type == SegmentType.proximal and self.menuShowProximalSynapsesPredicted.isChecked()):
          color = self.colorSynapsePredicted
        else:
          isVisible = False
      elif synapse.isConnected.atGivenStepAgo(Global.selStep):
        if (segment.type == SegmentType.proximal and self.menuShowProximalSynapsesConnected.isChecked()) or (segment.type == SegmentType.distal and self.menuShowDistalSynapsesConnected.isChecked()):
          color = self.colorSynapseConnected
        else:
          isVisible = False
      else:
        if (segment.type == SegmentType.proximal and self.menuShowProximalSynapsesActive.isChecked()) or (segment.type == SegmentType.distal and self.menuShowDistalSynapsesActive.isChecked()):
          color = self.colorInactive
        else:
          isVisible = False

    if isVisible and segmentIsVisible:
      # Draw the synapse
      if not synapse.tree3d_initialized:
        pts = numpy.array([[segment.tree3d_x2, segment.tree3d_y2, segment.tree3d_z2], [synapse.inputElem.tree3d_x, synapse.inputElem.tree3d_y, synapse.inputElem.tree3d_z]])
        synapse.tree3d_item = gl.GLLinePlotItem(pos=pts, width=1, antialias=False)
        synapse.tree3d_initialized = True
        self.simulationViewer.addItem(synapse.tree3d_item)

      # Update the color
      if synapse.tree3d_selected:
        color = self.colorSelected
      synapse.tree3d_item.color = pg.glColor(color)
    else:
      synapse.tree3d_initialized = False
      if synapse.tree3d_item in self.simulationViewer.items:
        self.simulationViewer.removeItem(synapse.tree3d_item)

  def selectView(self, viewMenu):
    """
    Load a pre-defined view and refresh controls.
    """

    if self.selectedViewMenu != None:
      self.selectedViewMenu.setChecked(False)
    self.selectedViewMenu = viewMenu
    self.selectedViewMenu.setChecked(True)

    # Find the specified view in the views list
    for view in Global.views:
      if view.menu == viewMenu:

        # Update menus
        self.menuShowBitsNone.setChecked(view.showBitsNone)
        self.menuShowBitsActive.setChecked(view.showBitsActive)
        self.menuShowBitsPredicted.setChecked(view.showBitsPredicted)
        self.menuShowBitsFalselyPredicted.setChecked(view.showBitsFalselyPredicted)
        self.menuShowCellsNone.setChecked(view.showCellsNone)
        self.menuShowCellsLearning.setChecked(view.showCellsLearning)
        self.menuShowCellsActive.setChecked(view.showCellsActive)
        self.menuShowCellsPredicted.setChecked(view.showCellsPredicted)
        self.menuShowCellsFalselyPredicted.setChecked(view.showCellsFalselyPredicted)
        self.menuShowCellsInactive.setChecked(view.showCellsInactive)
        self.menuShowProximalSegmentsNone.setChecked(view.showProximalSegmentsNone)
        self.menuShowProximalSegmentsActive.setChecked(view.showProximalSegmentsActive)
        self.menuShowProximalSegmentsPredicted.setChecked(view.showProximalSegmentsPredicted)
        self.menuShowProximalSegmentsFalselyPredicted.setChecked(view.showProximalSegmentsFalselyPredicted)
        self.menuShowProximalSynapsesNone.setChecked(view.showProximalSynapsesNone)
        self.menuShowProximalSynapsesConnected.setChecked(view.showProximalSynapsesConnected)
        self.menuShowProximalSynapsesActive.setChecked(view.showProximalSynapsesActive)
        self.menuShowProximalSynapsesPredicted.setChecked(view.showProximalSynapsesPredicted)
        self.menuShowProximalSynapsesFalselyPredicted.setChecked(view.showProximalSynapsesFalselyPredicted)
        self.menuShowDistalSegmentsNone.setChecked(view.showDistalSegmentsNone)
        self.menuShowDistalSegmentsActive.setChecked(view.showDistalSegmentsActive)
        self.menuShowDistalSynapsesNone.setChecked(view.showDistalSynapsesNone)
        self.menuShowDistalSynapsesConnected.setChecked(view.showDistalSynapsesConnected)
        self.menuShowDistalSynapsesActive.setChecked(view.showDistalSynapsesActive)

        # Update simulation
        self.refreshControls()

        break

    # Disable change options for the 'Default' view
    defaultViewSelected = False
    if self.selectedViewMenu == self.defaultViewMenu:
      defaultViewSelected = True
    self.menuViewsSave.setEnabled(not defaultViewSelected)
    self.menuViewsDelete.setEnabled(not defaultViewSelected)

  #endregion

  #region Events

  def menuLegend_Click(self, event):
    simulationLegendForm = SimulationLegendForm()
    simulationLegendForm.exec_()

  def menuShowBits_Click(self, event):
    menuClicked = self.sender()

    if menuClicked == self.menuShowBitsNone:
      if self.menuShowBitsNone.isChecked():
        self.menuShowBitsActive.setChecked(False)
        self.menuShowBitsPredicted.setChecked(False)
        self.menuShowBitsFalselyPredicted.setChecked(False)
    else:
      self.menuShowBitsNone.setChecked(False)

    self.refreshControls()

  def menuShowCells_Click(self, event):
    menuClicked = self.sender()

    if menuClicked == self.menuShowCellsNone:
      if self.menuShowCellsNone.isChecked():
        self.menuShowCellsLearning.setChecked(False)
        self.menuShowCellsActive.setChecked(False)
        self.menuShowCellsPredicted.setChecked(False)
        self.menuShowCellsFalselyPredicted.setChecked(False)
        self.menuShowCellsInactive.setChecked(False)
    else:
      self.menuShowCellsNone.setChecked(False)

    self.refreshControls()

  def menuShowProximalSegments_Click(self, event):
    menuClicked = self.sender()

    if menuClicked == self.menuShowProximalSegmentsNone:
      if self.menuShowProximalSegmentsNone.isChecked():
        self.menuShowProximalSegmentsActive.setChecked(False)
        self.menuShowProximalSegmentsPredicted.setChecked(False)
        self.menuShowProximalSegmentsFalselyPredicted.setChecked(False)
    else:
      self.menuShowProximalSegmentsNone.setChecked(False)

    self.refreshControls()

  def menuShowProximalSynapses_Click(self, event):
    menuClicked = self.sender()

    if menuClicked == self.menuShowProximalSynapsesNone:
      if self.menuShowProximalSynapsesNone.isChecked():
        self.menuShowProximalSynapsesConnected.setChecked(False)
        self.menuShowProximalSynapsesActive.setChecked(False)
        self.menuShowProximalSynapsesPredicted.setChecked(False)
        self.menuShowProximalSynapsesFalselyPredicted.setChecked(False)
    else:
      self.menuShowProximalSynapsesNone.setChecked(False)

    self.refreshControls()

  def menuShowDistalSegments_Click(self, event):
    menuClicked = self.sender()

    if menuClicked == self.menuShowDistalSegmentsNone:
      if self.menuShowDistalSegmentsNone.isChecked():
        self.menuShowDistalSegmentsActive.setChecked(False)
    else:
      self.menuShowDistalSegmentsNone.setChecked(False)

    self.refreshControls()

  def menuShowDistalSynapses_Click(self, event):
    menuClicked = self.sender()

    if menuClicked == self.menuShowDistalSynapsesNone:
      if self.menuShowDistalSynapsesNone.isChecked():
        self.menuShowDistalSynapsesConnected.setChecked(False)
        self.menuShowDistalSynapsesActive.setChecked(False)
    else:
      self.menuShowDistalSynapsesNone.setChecked(False)

    self.refreshControls()

  def menuView_Click(self, event):
    menuClicked = self.sender()
    self.selectView(menuClicked)

  def menuViewsNew_Click(self, event):

    # Ask for views's name
    enteredText, ok = QtGui.QInputDialog.getText(self, "Input Dialog", "Enter views' name:")
    if ok:
      view = View()
      view.name = enteredText
      view.menu = QtGui.QAction(self)
      view.menu.setText(view.name)
      view.menu.setCheckable(True)
      view.menu.triggered.connect(self.menuView_Click)

      Global.views.append(view)
      self.menuViews.addAction(view.menu)

      self.selectView(view.menu)

  def menuViewsSave_Click(self, event):

    # Find the specified view in the views list
    for view in Global.views:
      if view.menu == self.selectedViewMenu:
        view.showBitsNone = self.menuShowBitsNone.isChecked()
        view.showBitsActive = self.menuShowBitsActive.isChecked()
        view.showBitsPredicted = self.menuShowBitsPredicted.isChecked()
        view.showBitsFalselyPredicted = self.menuShowBitsFalselyPredicted.isChecked()
        view.showCellsNone = self.menuShowCellsNone.isChecked()
        view.showCellsLearning = self.menuShowCellsLearning.isChecked()
        view.showCellsActive = self.menuShowCellsActive.isChecked()
        view.showCellsPredicted = self.menuShowCellsPredicted.isChecked()
        view.showCellsFalselyPredicted = self.menuShowCellsFalselyPredicted.isChecked()
        view.showCellsInactive = self.menuShowCellsInactive.isChecked()
        view.showProximalSegmentsNone = self.menuShowProximalSegmentsNone.isChecked()
        view.showProximalSegmentsActive = self.menuShowProximalSegmentsActive.isChecked()
        view.showProximalSegmentsPredicted = self.menuShowProximalSegmentsPredicted.isChecked()
        view.showProximalSegmentsFalselyPredicted = self.menuShowProximalSegmentsFalselyPredicted.isChecked()
        view.showProximalSynapsesNone = self.menuShowProximalSynapsesNone.isChecked()
        view.showProximalSynapsesConnected = self.menuShowProximalSynapsesConnected.isChecked()
        view.showProximalSynapsesActive = self.menuShowProximalSynapsesActive.isChecked()
        view.showProximalSynapsesPredicted = self.menuShowProximalSynapsesPredicted.isChecked()
        view.showProximalSynapsesFalselyPredicted = self.menuShowProximalSynapsesFalselyPredicted.isChecked()
        view.showDistalSegmentsNone = self.menuShowDistalSegmentsNone.isChecked()
        view.showDistalSegmentsActive = self.menuShowDistalSegmentsActive.isChecked()
        view.showDistalSynapsesNone = self.menuShowDistalSynapsesNone.isChecked()
        view.showDistalSynapsesConnected = self.menuShowDistalSynapsesConnected.isChecked()
        view.showDistalSynapsesActive = self.menuShowDistalSynapsesActive.isChecked()

        Global.saveConfig()

        break

  def menuViewsDelete_Click(self, event):

    # Find the specified view in the views list
    for view in Global.views:
      if view.menu == self.selectedViewMenu:
        Global.views.remove(view)
        self.menuViews.removeAction(view.menu)
        break

    # Set 'Default' view as initial view
    self.selectView(self.defaultViewMenu)

  #endregion



from PyQt4 import QtGui, QtCore
from nupic_studio.ui import Global

class ProjectPropertiesForm(QtGui.QDialog):

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    QtGui.QDialog.__init__(self)

    self.initUI()

  #endregion

  #region Methods

  def initUI(self):

    # labelName
    self.labelName = QtGui.QLabel()
    self.labelName.setText("Name")
    self.labelName.setAlignment(QtCore.Qt.AlignRight)

    # textBoxName
    self.textBoxName = QtGui.QLineEdit()

    # labelAuthor
    self.labelAuthor = QtGui.QLabel()
    self.labelAuthor.setText("Author")
    self.labelAuthor.setAlignment(QtCore.Qt.AlignRight)

    # textBoxAuthor
    self.textBoxAuthor = QtGui.QLineEdit()

    # labelDescription
    self.labelDescription = QtGui.QLabel()
    self.labelDescription.setText("Description")
    self.labelDescription.setAlignment(QtCore.Qt.AlignRight)

    # textBoxDescription
    self.textBoxDescription = QtGui.QTextEdit()

    # groupBoxMainLayout
    groupBoxMainLayout = QtGui.QGridLayout()
    groupBoxMainLayout.addWidget(self.labelName, 0, 0)
    groupBoxMainLayout.addWidget(self.textBoxName, 0, 1)
    groupBoxMainLayout.addWidget(self.labelAuthor, 1, 0)
    groupBoxMainLayout.addWidget(self.textBoxAuthor, 1, 1)
    groupBoxMainLayout.addWidget(self.labelDescription, 2, 0)
    groupBoxMainLayout.addWidget(self.textBoxDescription, 2, 1)

    # groupBoxMain
    self.groupBoxMain = QtGui.QGroupBox()
    self.groupBoxMain.setLayout(groupBoxMainLayout)

    # buttonBox
    self.buttonBox = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Ok | QtGui.QDialogButtonBox.Cancel)
    self.buttonBox.button(QtGui.QDialogButtonBox.Ok).clicked.connect(self.__buttonOk_Click)
    self.buttonBox.button(QtGui.QDialogButtonBox.Cancel).clicked.connect(self.__buttonCancel_Click)

    # layout
    layout = QtGui.QVBoxLayout()
    layout.addWidget(self.groupBoxMain)
    layout.addWidget(self.buttonBox)

    # ProjectPropertiesForm
    self.setLayout(layout)
    self.setModal(True)
    self.setWindowTitle("Project Properties")
    self.setWindowIcon(QtGui.QIcon(Global.appPath + '/images/logo.ico'))
    self.resize(450, 200)

  def setControlsValues(self):
    """
    Set controls values from a class instance.
    """

    # Set controls values with project properties
    self.textBoxName.setText(Global.project.name)
    self.textBoxAuthor.setText(Global.project.author)
    self.textBoxDescription.setText(Global.project.description)

  #endregion

  #region Events

  def __buttonOk_Click(self, event):
    name = self.textBoxName.text()
    author = self.textBoxAuthor.text()
    description = self.textBoxDescription.toPlainText()

    # If anything has changed
    if Global.project.name != name or Global.project.author != author or Global.project.description != description:
      # Set project properties with controls values
      Global.project.name = name
      Global.project.author = author
      Global.project.description = description
      self.accept()

    self.close()

  def __buttonCancel_Click(self, event):
    self.reject()
    self.close()

  #endregion



import numpy
import time
from PyQt4 import QtGui, QtCore
from nupic_studio.htm import maxPreviousSteps
from nupic_studio.htm.node import Node, NodeType
from nupic_studio.htm.column import Column
from nupic_studio.htm.cell import Cell
from nupic_studio.htm.segment import Segment, SegmentType
from nupic_studio.htm.synapse import Synapse
from nupic_studio.ui import Global
from nupic.research.spatial_pooler import SpatialPooler
from nupic.research.temporal_memory import TemporalMemory as TemporalPooler

class Region(Node):
  """
  A class only to group properties related to regions.
  """

  #region Constructor

  def __init__(self, name):
    """
    Initializes a new instance of this class.
    """

    Node.__init__(self, name, NodeType.region)

    #region Instance fields

    self.columns = []
    """List of columns that compose this region"""

    self._inputMap = []
    """An array representing the input map for this region."""

    #region Spatial Parameters

    self.enableSpatialLearning = True
    """Switch for spatial learning"""

    self.potentialRadius = 0
    """This parameter determines the extent of the input that each column can potentially be connected to. This can be thought of as the input bits that are visible to each column, or a 'receptiveField' of the field of vision. A large enough value will result in 'global coverage', meaning that each column can potentially be connected to every input bit. This parameter defines a square (or hyper square) area: a column will have a max square potential pool with sides of length 2 * potentialRadius + 1."""

    self.potentialPct = 0.5
    """The percent of the inputs, within a column's potential radius, that a column can be connected to. If set to 1, the column will be connected to every input within its potential radius. This parameter is used to give each column a unique potential pool when a large potentialRadius causes overlap between the columns. At initialization time we choose ((2*potentialRadius + 1)^(# inputDimensions) * potentialPct) input bits to comprise the column's potential pool."""

    self.globalInhibition = False
    """If true, then during inhibition phase the winning columns are selected as the most active columns from the region as a whole. Otherwise, the winning columns are selected with respect to their local neighborhoods. Using global inhibition boosts performance x60."""

    self.localAreaDensity = -1.0
    """The desired density of active columns within a local inhibition area (the size of which is set by the internally calculated inhibitionRadius, which is in turn determined from the average size of the connected potential pools of all columns). The inhibition logic will insure that at most N columns remain ON within a local inhibition area, where N = localAreaDensity * (total number of columns in inhibition area)."""

    self.numActiveColumnsPerInhArea = int(0.02 * (self.width * self.height))
    """An alternate way to control the density of the active columns. If numActiveColumnsPerInhArea is specified then localAreaDensity must be less than 0, and vice versa. When using numActiveColumnsPerInhArea, the inhibition logic will insure that at most 'numActiveColumnsPerInhArea' columns remain ON within a local inhibition area (the size of which is set by the internally calculated inhibitionRadius, which is in turn determined from the average size of the connected receptive fields of all columns). When using this method, as columns learn and grow their effective receptive fields, the inhibitionRadius will grow, and hence the net density of the active columns will *decrease*. This is in contrast to the localAreaDensity method, which keeps the density of active columns the same regardless of the size of their receptive fields."""

    self.stimulusThreshold = 0
    """This is a number specifying the minimum number of synapses that must be on in order for a columns to turn ON. The purpose of this is to prevent noise input from activating columns. Specified as a percent of a fully grown synapse."""

    self.proximalSynConnectedPerm = 0.10
    """The default connected threshold. Any synapse whose permanence value is above the connected threshold is a "connected synapse", meaning it can contribute to the cell's firing."""

    self.proximalSynPermIncrement = 0.1
    """The amount by which an active synapse is incremented in each round. Specified as a percent of a fully grown synapse."""

    self.proximalSynPermDecrement = 0.01
    """The amount by which an inactive synapse is decremented in each round. Specified as a percent of a fully grown synapse."""

    self.minPctOverlapDutyCycle = 0.001
    """A number between 0 and 1.0, used to set a floor on how often a column should have at least stimulusThreshold active inputs. Periodically, each column looks at the overlap duty cycle of all other columns within its inhibition radius and sets its own internal minimal acceptable duty cycle to:
      minPctDutyCycleBeforeInh * max(other columns' duty cycles).
    On each iteration, any column whose overlap duty cycle falls below this computed value will get all of its permanence values boosted up by synPermActiveInc. Raising all permanences in response to a sub-par duty cycle before inhibition allows a cell to search for new inputs when either its previously learned inputs are no longer ever active, or when the vast majority of them have been "hijacked" by other columns."""

    self.minPctActiveDutyCycle = 0.001
    """A number between 0 and 1.0, used to set a floor on how often a column should be activate. Periodically, each column looks at the activity duty cycle of all other columns within its inhibition radius and sets its own internal minimal acceptable duty cycle to:
      minPctDutyCycleAfterInh * max(other columns' duty cycles).
    On each iteration, any column whose duty cycle after inhibition falls below this computed value will get its internal boost factor increased."""

    self.dutyCyclePeriod = 1000
    """The period used to calculate duty cycles. Higher values make it take longer to respond to changes in boost or synPerConnectedCell. Shorter values make it more unstable and likely to oscillate."""

    self.maxBoost = 10.0
    """The maximum overlap boost factor. Each column's overlap gets multiplied by a boost factor before it gets considered for inhibition. The actual boost factor for a column is number between 1.0 and maxBoost. A boost factor of 1.0 is used if the duty cycle is >= minOverlapDutyCycle, maxBoost is used if the duty cycle is 0, and any duty cycle in between is linearly extrapolated from these 2 endpoints."""

    self.spSeed = -1
    """Seed for generate random values"""

    #endregion

    #region Temporal Parameters

    self.enableTemporalLearning = True
    """Switch for temporal learning"""

    self.numCellsPerColumn = 10
    """Number of cells per column. More cells, more contextual information"""

    self.distalSynInitialPerm = 0.11
    """The initial permanence of an distal synapse."""

    self.distalSynConnectedPerm = 0.50
    """The default connected threshold. Any synapse whose permanence value is above the connected threshold is a "connected synapse", meaning it can contribute to the cell's firing."""

    self.distalSynPermIncrement = 0.10
    """The amount by which an active synapse is incremented in each round. Specified as a percent of a fully grown synapse."""

    self.distalSynPermDecrement = 0.10
    """The amount by which an inactive synapse is decremented in each round. Specified as a percent of a fully grown synapse."""

    self.minThreshold = 8
    """If the number of synapses active on a segment is at least this threshold, it is selected as the best matching cell in a bursing column."""

    self.activationThreshold = 12
    """If the number of active connected synapses on a segment is at least this threshold, the segment is said to be active."""

    self.maxNumNewSynapses = 15
    """The maximum number of synapses added to a segment during learning."""

    self.tpSeed = 42
    """Seed for generate random values"""

    #endregion

    self.spatialPooler = None
    """Spatial Pooler instance"""

    self.temporalPooler = None
    """Temporal Pooler instance"""

    #endregion

    #region Statistics properties

    self.statsPrecisionRate = 0.

    #endregion

  #endregion

  #region Methods

  def getColumn(self, x, y):
    """
    Return the column located at given position
    """

    column = self.columns[(y * self.width) + x]

    return column

  def getInputSize(self):
    """
    Return the sum of sizes of all feeder nodes.
    """

    sumSizes = 0
    for feeder in Global.project.network.getFeederNodes(self):
      sumSizes += feeder.width * feeder.height

    return sumSizes

  def initialize(self):
    """
    Initialize this node.
    """

    # Check if this region has nodes that feed it
    numFeeders = len(Global.project.network.getFeederNodes(self))
    if numFeeders == 0:
      QtGui.QMessageBox.warning(None, "Warning", "Region '" + self.name + "' does not have any child!")
      return

    # Initialize this node and the nodes that feed it
    Node.initialize(self)

    # Create the input map
    # An input map is a set of input elements (cells or sensor bits) that should are grouped
    # For example, if we have 2 nodes that feed this region (#1 and #2) with dimensions 6 and 12 respectively,
    # a input map would be something like:
    #   111111222222222222
    self._inputMap = []
    elemIdx = 0
    for feeder in Global.project.network.getFeederNodes(self):

      # Arrange input from feeder into input map of this region
      if feeder.type == NodeType.region:
        for column in feeder.columns:
          inputElem = column.cells[0]
          self._inputMap.append(inputElem)
      else:
        for bit in feeder.bits:
          inputElem = bit
          self._inputMap.append(inputElem)
      elemIdx += 1

    # Initialize elements
    self.columns = []
    colIdx = 0
    for x in range(self.width):
      for y in range(self.height):
        column = Column()
        column.x = x
        column.y = y
        for z in range(self.numCellsPerColumn):
          cell = Cell()
          cell.index = (colIdx * self.numCellsPerColumn) + z
          cell.z = z
          column.cells.append(cell)
        self.columns.append(column)
        colIdx += 1

    # Create Spatial Pooler instance with appropriate parameters
    self.spatialPooler = SpatialPooler(
      inputDimensions = (self.getInputSize(), 1),
      columnDimensions = (self.width, self.height),
      potentialRadius = self.potentialRadius,
      potentialPct = self.potentialPct,
      globalInhibition = self.globalInhibition,
      localAreaDensity = self.localAreaDensity,
      numActiveColumnsPerInhArea = self.numActiveColumnsPerInhArea,
      stimulusThreshold = self.stimulusThreshold,
      synPermInactiveDec = self.proximalSynPermDecrement,
      synPermActiveInc = self.proximalSynPermIncrement,
      synPermConnected = self.proximalSynConnectedPerm,
      minPctOverlapDutyCycle = self.minPctOverlapDutyCycle,
      minPctActiveDutyCycle = self.minPctActiveDutyCycle,
      dutyCyclePeriod = self.dutyCyclePeriod,
      maxBoost = self.maxBoost,
      seed = self.spSeed,
      spVerbosity = False)

    # Create Temporal Pooler instance with appropriate parameters
    self.temporalPooler = TemporalPooler(
      columnDimensions = (self.width, self.height),
      cellsPerColumn = self.numCellsPerColumn,
      initialPermanence = self.distalSynInitialPerm,
      connectedPermanence = self.distalSynConnectedPerm,
      minThreshold = self.minThreshold,
      maxNewSynapseCount = self.maxNumNewSynapses,
      permanenceIncrement = self.distalSynPermIncrement,
      permanenceDecrement = self.distalSynPermDecrement,
      activationThreshold = self.activationThreshold,
      seed = self.tpSeed)

    return True

  def nextStep(self):
    """
    Perfoms actions related to time step progression.
    """

    Node.nextStep(self)
    for column in self.columns:
      column.nextStep()

    # Get input from sensors or lower regions and put into a single input map.
    input = self.getInput()

    # Send input to Spatial Pooler and get processed output (i.e. the active columns)
    # First initialize the vector for representing the current record
    columnDimensions = (self.width, self.height)
    columnNumber = numpy.array(columnDimensions).prod()
    activeColumns = numpy.zeros(columnNumber)
    self.spatialPooler.compute(input, self.enableSpatialLearning, activeColumns)

    # Send active columns to Temporal Pooler and get processed output (i.e. the predicting cells)
    # First convert active columns from float array to integer set
    activeColumnsSet = set()
    for colIdx in range(len(activeColumns)):
      if activeColumns[colIdx] == 1:
        activeColumnsSet.add(colIdx)
    self.temporalPooler.compute(activeColumnsSet, self.enableTemporalLearning)

    # Update elements regarding spatial pooler
    self.updateSpatialElements(activeColumns)

    # Update elements regarding temporal pooler
    self.updateTemporalElements()

    # Get the predicted values
    self.getPredictions()

    #TODO: self._output = self.temporalPooler.getPredictedState()

  def getPredictions(self):
    """
    Get the predicted values after an iteration.
    """

    for feeder in Global.project.network.getFeederNodes(self):
      feeder.getPredictions()

  def calculateStatistics(self):
    """
    Calculate statistics after an iteration.
    """

    # The region's prediction precision is the average between the nodes that feed it
    precisionRate = 0.
    numFeeders = 0
    for feeder in Global.project.network.getFeederNodes(self):
      precisionRate += feeder.statsPrecisionRate
      numFeeders += 1
    self.statsPrecisionRate = precisionRate / numFeeders

    for column in self.columns:
      column.calculateStatistics()

  def getInput(self):
    """
    Get input from sensors or lower regions and put into a single input map.
    """

    # Initialize the vector for representing the current input map
    inputList = []
    for inputElem in self._inputMap:
      if inputElem.isActive.atCurrStep():
        inputList.append(1)
      else:
        inputList.append(0)
    input = numpy.array(inputList)

    return input

  def updateSpatialElements(self, activeColumns):
    """
    Update elements regarding spatial pooler
    """

    # Update proximal segments and synapses according to active columns
    for colIdx in range(len(self.columns)):
      column = self.columns[colIdx]

      # Update proximal segment
      segment = column.segment
      if activeColumns[colIdx] == 1:
        segment.isActive.setForCurrStep(True)
      else:
        segment.isActive.setForCurrStep(False)

      # Check if proximal segment is predicted by check if the column has any predicted cell
      for cell in column.cells:
        if cell.index in self.temporalPooler.predictiveCells:
          segment.isPredicted.setForCurrStep(True)

      # Update proximal synapses
      if segment.isActive.atCurrStep() or segment.isPredicted.atCurrStep():
        permanencesSynapses = []
        self.spatialPooler.getPermanence(colIdx, permanencesSynapses)
        connectedSynapses = []
        self.spatialPooler.getConnectedSynapses(colIdx, connectedSynapses)
        for synIdx in range(len(permanencesSynapses)):
          # Get the proximal synapse given its position in the input map
          # Create a new one if it doesn't exist
          synapse = segment.getSynapse(synIdx)

          # Update proximal synapse
          if permanencesSynapses[synIdx] > 0.:
            if synapse == None:
              # Create a new synapse to a input element
              # An input element is a column if feeder is a region
              # or then a bit if feeder is a sensor
              synapse = Synapse()
              synapse.inputElem = self._inputMap[synIdx]
              synapse.indexSP = synIdx
              segment.synapses.append(synapse)

            # Update state
            synapse.isRemoved.setForCurrStep(False)
            synapse.permanence.setForCurrStep(permanencesSynapses[synIdx])
            if connectedSynapses[synIdx] == 1:
              synapse.isConnected.setForCurrStep(True)
            else:
              synapse.isConnected.setForCurrStep(False)
          else:
            if synapse != None:
              synapse.isRemoved.setForCurrStep(True)

  def updateTemporalElements(self):
    """
    Update elements regarding temporal pooler
    """

    # Update cells, distal segments and synapses according to active columns
    for colIdx in range(len(self.columns)):
      column = self.columns[colIdx]

      # Mark proximal segment and its connected synapses as predicted
      if column.segment.isPredicted.atCurrStep():
        for synapse in column.segment.synapses:
          if synapse.isConnected.atCurrStep():
            synapse.isPredicted.setForCurrStep(True)
            synapse.inputElem.isPredicted.setForCurrStep(True)

      # Mark proximal segment and its connected synapses that were predicted but are not active now
      if column.segment.isPredicted.atPreviousStep():
        if not column.segment.isActive.atCurrStep():
          column.segment.isFalselyPredicted.setForCurrStep(True)
        for synapse in column.segment.synapses:
          if (synapse.isPredicted.atPreviousStep() and not synapse.isConnected.atCurrStep()) or (synapse.isConnected.atCurrStep() and synapse.inputElem.isFalselyPredicted.atCurrStep()):
            synapse.isFalselyPredicted.setForCurrStep(True)

      for cell in column.cells:
        cellIdx = cell.index

        # Update cell's states
        if cellIdx in self.temporalPooler.winnerCells:
          cell.isLearning.setForCurrStep(True)
        if cellIdx in self.temporalPooler.activeCells:
          cell.isActive.setForCurrStep(True)
        if cellIdx in self.temporalPooler.predictiveCells:
          cell.isPredicted.setForCurrStep(True)
        if cell.isPredicted.atPreviousStep() and not cell.isActive.atCurrStep():
          cell.isFalselyPredicted.setForCurrStep(True)

        # Get the indexes of the distal segments of this cell
        segmentsForCell = self.temporalPooler.connections.segmentsForCell(cellIdx)

        # Add the segments that appeared after last iteration
        for segIdx in segmentsForCell:
          # Check if segment already exists in the cell
          segFound = False
          for segment in cell.segments:
            if segment.indexTP == segIdx:
              segFound = True
              break

          # If segment is new, add it to cell
          if not segFound:
            segment = Segment(SegmentType.distal)
            segment.indexTP = segIdx
            cell.segments.append(segment)

        # Update distal segments
        for segment in cell.segments:
          segIdx = segment.indexTP

          # If segment not found in segments indexes returned in last iteration mark it as removed
          if segIdx in segmentsForCell:

            # Update segment's state
            if segIdx in self.temporalPooler.activeSegments:
              segment.isActive.setForCurrStep(True)
            else:
              segment.isActive.setForCurrStep(False)

            # Get the indexes of the synapses of this segment
            synapsesForSegment = self.temporalPooler.connections.synapsesForSegment(segIdx)

            # Add the synapses that appeared after last iteration
            for synIdx in synapsesForSegment:
              # Check if synapse already exists in the segment
              synFound = False
              for synapse in segment.synapses:
                if synapse.indexTP == synIdx:
                  synFound = True
                  break

              # If synapse is new, add it to segment
              if not synFound:
                synapse = Synapse()
                synapse.indexTP = synIdx
                segment.synapses.append(synapse)

            # Update synapses
            for synapse in segment.synapses:
              synIdx = synapse.indexTP

              # If synapse not found in synapses indexes returned in last iteration mark it as removed
              if synIdx in synapsesForSegment:

                # Update synapse's state
                (_, sourceCellAbsIdx, permanence) = self.temporalPooler.connections.dataForSynapse(synIdx)
                synapse.permanence.setForCurrStep(permanence)
                if permanence >= self.distalSynConnectedPerm:
                  synapse.isConnected.setForCurrStep(True)
                else:
                  synapse.isConnected.setForCurrStep(False)

                # Get cell given cell's index
                sourceColIdx = sourceCellAbsIdx / self.numCellsPerColumn
                sourceCellRelIdx = sourceCellAbsIdx % self.numCellsPerColumn
                sourceCell = self.columns[sourceColIdx].cells[sourceCellRelIdx]
                synapse.inputElem = sourceCell
              else:
                synapse.isRemoved.setForCurrStep(True)
          else:
            segment.isRemoved.setForCurrStep(True)

  #endregion
